@@ -1,14 +1,19 @@
 package aQute.lib.index;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
 import java.io.File;
 
+import org.junit.jupiter.api.Test;
+
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.lib.io.IO;
-import junit.framework.TestCase;
 
-public class TestIndex extends TestCase {
+public class TestIndex {
 
-	public void testPersistence() throws Exception {
-		File test = IO.getFile("tmp/" + getName() + ".tmp");
+	@Test
+	public void testPersistence(@InjectTemporaryDirectory
+	File test) throws Exception {
 		test.delete();
 		test.getParentFile()
 			.mkdirs();
@@ -58,8 +63,9 @@ public class TestIndex extends TestCase {
 		}
 	}
 
-	public void testBasic() throws Exception {
-		File test = IO.getFile("tmp/" + getName() + ".tmp");
+	@Test
+	public void testBasic(@InjectTemporaryDirectory
+	File test) throws Exception {
 		test.delete();
 		test.getParentFile()
 			.mkdirs();
@@ -96,18 +102,4 @@ public class TestIndex extends TestCase {
 		}
 	}
 
-	// public void testMany() throws Exception {
-	// File test = IO.getFile("tmp/"+getName()+".tmp");
-	// test.delete();
-	// Index index = new Index(test, 1000);
-	// for ( int i = 1; i<127; i++)
-	// index.insert( new byte[]{(byte)i}, i*2);
-	//
-	// index.close();
-	// Index index2 = new Index(test, 1000);
-	// for ( int i = 1; i<127; i++)
-	// assertEquals(i*2, index2.search( new byte[(byte)i]));
-	//
-	// }
-
 }
