@@ -64,4 +64,14 @@ public interface EntryNamesReference {
 	 * The version, the artifactId and the groupId of the analyzed object.
 	 */
 	String	MAVEN_COORDINATE	= "mavenCoordinate";
+
+	/**
+	 * A list of gogo-commands.
+	 */
+	String	GOGO_COMMANDS		= "gogoCommands";
+
+	/**
+	 * The checksum, of the bundle / ouptut file.
+	 */
+	String	CHECKSUM			= "checksum";
 }
