@@ -1,9 +1,9 @@
 package biz.aQute.launcher;
 
 import static org.assertj.core.api.Assertions.assertThat;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.ByteArrayOutputStream;
 import java.io.File;
@@ -16,10 +16,7 @@ import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.nio.charset.StandardCharsets;
-import java.nio.file.Path;
-import java.nio.file.Paths;
-import java.time.ZoneOffset;
-import java.time.format.DateTimeFormatter;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Map.Entry;
@@ -31,12 +28,16 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.assertj.core.api.AutoCloseableSoftAssertions;
+import org.assertj.core.api.SoftAssertions;
+import org.assertj.core.api.junit.jupiter.InjectSoftAssertions;
+import org.assertj.core.api.junit.jupiter.SoftAssertionsExtension;
 import org.assertj.core.util.Files;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TestName;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.extension.ExtendWith;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
 
 import aQute.bnd.build.Container;
 import aQute.bnd.build.Project;
@@ -45,31 +46,34 @@ import aQute.bnd.build.ProjectLauncher.NotificationType;
 import aQute.bnd.build.ProjectTester;
 import aQute.bnd.build.Run;
 import aQute.bnd.build.Workspace;
+import aQute.bnd.header.Parameters;
 import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.Domain;
 import aQute.bnd.osgi.Jar;
 import aQute.bnd.osgi.Macro;
 import aQute.bnd.osgi.Resource;
 import aQute.bnd.service.Strategy;
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.lib.io.IO;
+import aQute.lib.strings.Strings;
 import aQute.libg.command.Command;
+import biz.aQute.launcher.LauncherTest.ArgumentRendering;
 import biz.aQute.resolve.Bndrun;
 
+@ExtendWith(SoftAssertionsExtension.class)
 public class AlsoLauncherTest {
-	public static final String	TMPDIR		= "generated/tmp/test";
-	@Rule
-	public final TestName		testName	= new TestName();
-	private File				testDir;
+	@InjectTemporaryDirectory
+	File				testDir;
 
-	private Workspace			workspace;
-	private Project				project;
-	private Properties			prior;
+	@InjectSoftAssertions
+	SoftAssertions		softly;
 
-	@Before
+	private Workspace	workspace;
+	private Project		project;
+	private Properties	prior;
+
+	@BeforeEach
 	public void setUp() throws Exception {
-		testDir = new File(TMPDIR, getClass().getName() + "/" + testName.getMethodName());
-		IO.delete(testDir);
-		IO.mkdirs(testDir);
 		prior = new Properties();
 		prior.putAll(System.getProperties());
 
@@ -83,80 +87,22 @@ public class AlsoLauncherTest {
 		IO.delete(new File(wsRoot, "cnf/cache"));
 		workspace = new Workspace(wsRoot);
 		workspace.setProperty(Constants.PROFILE, "prod");
+		workspace.setProperty(Constants.CONNECTION_SETTINGS, "false");
 		project = workspace.getProject("demo");
 		project.setTrace(true);
 		assertTrue(project.check());
 	}
 
 	@SuppressWarnings("restriction")
-	@After
+	@AfterEach
 	public void tearDown() throws Exception {
 		IO.close(project);
 		IO.close(workspace);
 		System.setProperties(prior);
 	}
 
-	@Test
-	public void testLocationFormat() throws Exception {
-		project.setProperty(Constants.RUNPROPERTIES, "test.cmd=exit");
-		project.setProperty(Constants.RUNTRACE, "true");
-		project.setProperty("-executable", "location='${@bsn}'");
-		Entry<String, Resource> export = project.export("bnd.executablejar", null);
-		assertThat(export).isNotNull();
-
-		try (Jar jar = new Jar(".", export.getValue()
-			.openInputStream())) {
-
-			assertThat(jar.getResources()
-				.keySet()).contains(//
-					"jar/biz.aQute.launcher.jar", // -runpath
-					"jar/org.apache.felix.framework-5.6.10.jar", // -runpath
-					"jar/apiguardian-api-1.1.0.jar", // not a bundle
-					"jar/demo", //
-					// the following were not a bundle yet
-					// "jar/junit-jupiter-api", //
-					// "jar/junit-jupiter-engine", //
-					// "jar/junit-jupiter-params", //
-					// "jar/junit-platform-commons", //
-					// "jar/junit-platform-engine", //
-					// "jar/junit-vintage-engine", //
-					"jar/org.apache.felix.configadmin", //
-					"jar/org.apache.felix.scr", //
-					"jar/org.apache.servicemix.bundles.junit", //
-					"jar/org.opentest4j" //
-			);
-
-			File tmp = File.createTempFile("foo", ".jar");
-			try {
-
-				jar.write(tmp);
-				Command cmd = new Command();
-				cmd.add(project.getJavaExecutable("java"));
-				cmd.add("-jar");
-				cmd.add(tmp.getAbsolutePath());
-
-				StringBuilder stdout = new StringBuilder();
-				StringBuilder stderr = new StringBuilder();
-				int execute = cmd.execute(stdout, stderr);
-				String output = stdout.append(stderr)
-					.toString();
-				System.out.println(output);
-
-				// These must be bsns ow
-				assertThat(output).contains("installing jar/org.apache.felix.scr",
-					"installing jar/org.apache.felix.configadmin");
-
-				assertThat(execute).isEqualTo(42);
-
-			} finally {
-				tmp.delete();
-			}
-		}
-	}
-
 	/**
 	 * Test that the Bndrun file is loaded when we create a run
-	 *
 	 */
 	@Test
 	public void testCreateBndrun() throws Exception {
@@ -186,6 +132,101 @@ public class AlsoLauncherTest {
 
 	}
 
+	@Test
+	public void testExportRunBundlesWithPath() throws Exception {
+		project.setProperty("-resourceonly", "true");
+		project.setProperty("-includeresource", "hello;literal=true");
+
+		project.setProperty("-export", "x.bndrun;type=bnd.runbundles;name=runbundles.jar;targetDir=test");
+		File file = project.getFile("generated/demo.jar");
+		file.delete();
+		assertThat(file).doesNotExist();
+
+		File[] build = project.build();
+		assertThat(project.check()).isTrue();
+		assertThat(build).hasSize(2);
+		assertThat(build[0]).isEqualTo(file);
+
+		File run = project.getFile("generated/runbundles.jar");
+		assertThat(build[1]).isEqualTo(run);
+
+		try (Jar jar = new Jar(run)) {
+			jar.getResources()
+				.forEach((k, v) -> assertThat(k).startsWith("test/"));
+		}
+	}
+
+	@Test
+	public void testExportRunBundlesTemplate() throws Exception {
+		project.setProperty("-resourceonly", "true");
+		project.setProperty("-includeresource", "hello;literal=true");
+
+		project.setProperty("-export", "x.bndrun;type=bnd.runbundles;name=runbundles.jar;template=" + project.getBase()
+			.getAbsolutePath() + "/testresources/biz.aQute.launcher-4.1.0.jar");
+		File file = project.getFile("generated/demo.jar");
+		file.delete();
+		assertThat(file).doesNotExist();
+
+		File[] build = project.build();
+		assertThat(project.check()).isTrue();
+		assertThat(build).hasSize(2);
+		assertThat(build[0]).isEqualTo(file);
+
+		File run = project.getFile("generated/runbundles.jar");
+		assertThat(build[1]).isEqualTo(run);
+
+		try (Jar jar = new Jar(run)) {
+			assertThat(jar.getResource("META-INF/maven/biz.aQute.bnd/biz.aQute.launcher/pom.xml")).isNotNull();
+		}
+	}
+
+	@Test
+	public void testExportRunBundlesTemplateCheckTemplateClosed() throws Exception {
+		project.setProperty("-resourceonly", "true");
+		project.setProperty("-includeresource", "hello;literal=true");
+
+		project.setProperty("-export", "x.bndrun;type=bnd.runbundles;name=runbundles.jar;template=" + project.getBase()
+			.getAbsolutePath() + "/generated/demo.jar");
+		File file = project.getFile("generated/demo.jar");
+		file.delete();
+		assertThat(file).doesNotExist();
+
+		File[] build = project.build();
+		assertThat(project.check()).isTrue();
+		assertThat(build).hasSize(2);
+		assertThat(build[0]).isEqualTo(file);
+
+		File run = project.getFile("generated/runbundles.jar");
+		assertThat(build[1]).isEqualTo(run);
+
+		file = project.getFile("generated/demo.jar");
+		file.delete();
+	}
+
+	@Test
+	public void testExportRunBundlesPlain() throws Exception {
+		project.setProperty("-resourceonly", "true");
+		project.setProperty("-includeresource", "hello;literal=true");
+
+		project.setProperty("-export", "x.bndrun;type=bnd.runbundles;name=runbundles.jar");
+		File file = project.getFile("generated/demo.jar");
+		file.delete();
+		assertThat(file).doesNotExist();
+
+		File[] build = project.build();
+		assertThat(project.check()).isTrue();
+		assertThat(build).hasSize(2);
+		assertThat(build[0]).isEqualTo(file);
+
+		File run = project.getFile("generated/runbundles.jar");
+		assertThat(build[1]).isEqualTo(run);
+
+		try (Jar jar = new Jar(run)) {
+			jar.getResources()
+				.forEach((k, v) -> assertThat(k).startsWith("org.apache"));
+		}
+	}
+
 	@Test
 	public void testExportOptions() throws Exception {
 		project.setProperty("-resourceonly", "true");
@@ -205,7 +246,6 @@ public class AlsoLauncherTest {
 
 		File[] build = project.build();
 
-
 		assertThat(project.check("Duplicate file in -export")).isTrue();
 		assertThat(build).hasSize(8);
 
@@ -283,6 +323,58 @@ public class AlsoLauncherTest {
 		}
 	}
 
+	@Test
+	public void testLocationFormat() throws Exception {
+		project.setProperty(Constants.RUNPROPERTIES, "test.cmd=exit");
+		project.setProperty(Constants.RUNTRACE, "true");
+		project.setProperty("-executable", "location='${@bsn}");
+		Entry<String, Resource> export = project.export("bnd.executablejar", null);
+		assertThat(project.check()).isTrue();
+		assertThat(export).isNotNull();
+
+		try (Jar jar = new Jar(".", export.getValue()
+			.openInputStream())) {
+
+			assertThat(jar.getResources()
+				.keySet()).contains(//
+					"jar/biz.aQute.launcher.jar", // -runpath
+					"jar/org.apache.felix.framework-5.6.10.jar", // -runpath
+					"jar/org.apiguardian.api", //
+					"jar/demo", //
+					"jar/junit-jupiter-api", //
+					"jar/junit-jupiter-engine", //
+					"jar/junit-jupiter-params", //
+					"jar/junit-platform-commons", //
+					"jar/junit-platform-engine", //
+					"jar/junit-vintage-engine", //
+					"jar/org.apache.felix.configadmin", //
+					"jar/org.apache.felix.scr", //
+					"jar/org.apache.servicemix.bundles.junit", //
+					"jar/org.opentest4j" //
+			);
+
+			File tmp = File.createTempFile("foo", ".jar", testDir);
+			jar.write(tmp);
+			Command cmd = new Command();
+			cmd.add(project.getJavaExecutable("java"));
+			cmd.add("-jar");
+			cmd.add(tmp.getAbsolutePath());
+
+			StringBuilder stdout = new StringBuilder();
+			StringBuilder stderr = new StringBuilder();
+			int execute = cmd.execute(stdout, stderr);
+			String output = stdout.append(stderr)
+				.toString();
+			System.out.println(output);
+
+			// These must be bsns ow
+			assertThat(output).contains("installing jar/org.apache.felix.scr",
+				"installing jar/org.apache.felix.configadmin");
+
+			assertThat(execute).isEqualTo(42);
+		}
+	}
+
 	@Test
 	public void testExecutableJarWithStripping() throws Exception {
 		long full = make(project, null);
@@ -396,22 +488,139 @@ public class AlsoLauncherTest {
 	@Test
 	public void testExpandedJarLauncher() throws Exception {
 		project.setProperty(Constants.RUNPROPERTIES, "test.cmd=quit.no.exit");
+		File temporaryFolder = new File(testDir, "executable");
 		ProjectLauncher l = project.getProjectLauncher();
-		File temporaryFolder = Files.newTemporaryFolder();
-		try {
-			try (Jar executable = l.executable()) {
-				executable.writeFolder(temporaryFolder);
+		try (Jar executable = l.executable()) {
+			executable.writeFolder(temporaryFolder);
+		}
+		try (URLClassLoader loader = new URLClassLoader(new URL[] {
+			temporaryFolder.toURI()
+				.toURL()
+		}, null)) {
+			Class<?> launcher = loader.loadClass("aQute.launcher.pre.EmbeddedLauncher");
+			Method method = launcher.getMethod("main", String[].class);
+			method.invoke(null, (Object) new String[0]);
+		}
+	}
+
+	static Pattern JAVACMD = Pattern.compile("java (.*) aQute.launcher.pre.EmbeddedLauncher([^\\n]*)$");
+
+	@Test
+	public void expandedJarLauncher_rendersVMAndProgramArgs() throws Exception {
+		project.setProperty("-runvm.test1", "-Dfoo=\"xyz abc\"");
+		project.setProperty("-runvm.test2",
+			"-Dbar=xyz abc, --add-opens=something, -Dlauncherer.properties=C:\\Users\\bj hargrave\\git\\bnd\\bndtools.core\\generated\\launch7354374140259840283.properties");
+		project.setProperty("-runprogramargs.test1", "123\\456");
+		project.setProperty("-runprogramargs.test2", "234, sdfs");
+
+		Parameters hdr = project.getMergedParameters(Constants.RUNVM);
+		List<String> expectedArgs = new ArrayList<>();
+		expectedArgs.add("-cp");
+		expectedArgs.add(".");
+		expectedArgs.addAll(hdr.keyList());
+
+		List<String> expectedProgramArgs = project.getMergedParameters(Constants.RUNPROGRAMARGS)
+			.keyList();
+
+		File temporaryFolder = new File(testDir, "executable");
+		ProjectLauncher l = project.getProjectLauncher();
+		try (Jar executable = l.executable()) {
+			executable.writeFolder(temporaryFolder);
+			final File start = new File(temporaryFolder, "start");
+			softly.assertThat(start)
+				.as("start")
+				.exists();
+			if (softly.wasSuccess()) {
+				String startContent = Files.contentOf(start, "UTF-8");
+				Matcher m = JAVACMD.matcher(startContent);
+				if (m.find()) {
+					final String vmArgs = m.group(1);
+					final String programArgs = m.group(2);
+					softly.assertThat(ArgumentRendering.parseArgumentsImpl(vmArgs, false))
+						.as("unix:vmArgs")
+						.containsExactlyElementsOf(expectedArgs);
+					softly.assertThat(programArgs)
+						.as("unix:progArgs")
+						.startsWith(" ");
+					softly.assertThat(ArgumentRendering.parseArgumentsImpl(programArgs.trim(), false))
+						.as("unix:trimmedProgramArgs")
+						.containsExactlyElementsOf(expectedProgramArgs);
+				} else {
+					softly.fail("unix: File content <%s> didn't match %s", startContent, JAVACMD);
+				}
 			}
-			try (URLClassLoader loader = new URLClassLoader(new URL[] {
-				temporaryFolder.toURI()
-					.toURL()
-			}, null)) {
-				Class<?> launcher = loader.loadClass("aQute.launcher.pre.EmbeddedLauncher");
-				Method method = launcher.getMethod("main", String[].class);
-				method.invoke(null, (Object) new String[0]);
+			final File startBat = new File(temporaryFolder, "start.bat");
+			softly.assertThat(startBat)
+				.as("start.bat")
+				.exists();
+			if (softly.wasSuccess()) {
+				String startContent = Files.contentOf(startBat, "UTF-8");
+				Matcher m = JAVACMD.matcher(startContent);
+				if (m.find()) {
+					final String vmArgs = m.group(1);
+					final String programArgs = m.group(2);
+					softly.assertThat(ArgumentRendering.parseArgumentsWindows(vmArgs, false))
+						.as("windows:vmArgs")
+						.containsExactlyElementsOf(expectedArgs);
+					softly.assertThat(programArgs)
+						.as("windows:progArgs")
+						.startsWith(" ");
+					softly.assertThat(ArgumentRendering.parseArgumentsWindows(programArgs.trim(), false))
+						.as("windows:trimmedProgramArgs")
+						.containsExactlyElementsOf(expectedProgramArgs);
+				} else {
+					softly.fail("windows: File content <%s> didn't match %s", startContent, JAVACMD);
+				}
+			}
+		}
+	}
+
+	@Test
+	public void expandedJarLauncher_handlesEmptyVMAndProgramArgs() throws Exception {
+		File temporaryFolder = new File(testDir, "executable");
+		ProjectLauncher l = project.getProjectLauncher();
+		try (Jar executable = l.executable()) {
+			executable.writeFolder(temporaryFolder);
+			final File start = new File(temporaryFolder, "start");
+			softly.assertThat(start)
+				.as("start")
+				.exists();
+			if (softly.wasSuccess()) {
+				String startContent = Files.contentOf(start, "UTF-8");
+				Matcher m = JAVACMD.matcher(startContent);
+				if (m.find()) {
+					final String vmArgs = m.group(1);
+					final String programArgs = m.group(2);
+					softly.assertThat(vmArgs.trim())
+						.as("unix:vmArgs")
+						.isEqualTo("-cp .");
+					softly.assertThat(programArgs.trim())
+						.as("unix:progArgs")
+						.isEmpty();
+				} else {
+					softly.fail("unix: File content <%s> didn't match %s", startContent, JAVACMD);
+				}
+			}
+			final File startBat = new File(temporaryFolder, "start.bat");
+			softly.assertThat(startBat)
+				.as("start.bat")
+				.exists();
+			if (softly.wasSuccess()) {
+				String startContent = Files.contentOf(startBat, "UTF-8");
+				Matcher m = JAVACMD.matcher(startContent);
+				if (m.find()) {
+					final String vmArgs = m.group(1);
+					final String programArgs = m.group(2);
+					softly.assertThat(vmArgs.trim())
+						.as("windows:vmArgs")
+						.isEqualTo("-cp .");
+					softly.assertThat(programArgs.trim())
+						.as("windows:progArgs")
+						.isEmpty();
+				} else {
+					softly.fail("windows: File content <%s> didn't match %s", startContent, JAVACMD);
+				}
 			}
-		} finally {
-			IO.delete(temporaryFolder);
 		}
 	}
 
@@ -726,63 +935,47 @@ public class AlsoLauncherTest {
 		}
 	}
 
-	@Test
-	public void testReporting_notUsingReferences() throws Exception {
+	@ParameterizedTest(name = "-runnoreferences={0}")
+	@ValueSource(strings = {
+		"true", "false"
+	})
+	public void testReporting_runnoreferences(String runnoreferences) throws Exception {
 		try (ProjectLauncher l = project.getProjectLauncher()) {
-			project.setProperty("-runnoreferences", "true");
-			doTestReporting(l);
-		}
-	}
+			project.setProperty("-runnoreferences", runnoreferences);
+			l.setTrace(true);
+			StringBuilder out = new StringBuilder();
+			l.setStreams(out, out);
+			l.getRunProperties()
+				.put("test.cmd", "framework.stop");
 
-	@Test
-	public void testReporting_usingReferences() throws Exception {
-		try (ProjectLauncher l = project.getProjectLauncher()) {
-			project.setProperty("-runnoreferences", "false");
-			doTestReporting(l);
-		}
-	}
+			try (AutoCloseableSoftAssertions softly = new AutoCloseableSoftAssertions()) {
+				Macro r = project.getReplacer();
 
-	private void doTestReporting(ProjectLauncher l) throws Exception {
-		l.setTrace(true);
-		StringBuilder out = new StringBuilder();
-		StringBuilder err = new StringBuilder();
-		l.setStreams(out, err);
-		l.getRunProperties()
-			.put("test.cmd", "framework.stop");
+				l.launch();
 
-		try (AutoCloseableSoftAssertions softly = new AutoCloseableSoftAssertions()) {
-			Macro r = project.getReplacer();
+				// High-level test; cuts down our search space for subsequent
+				// tests which makes the output more readable in the event of a
+				// subsequent assertion failure. Don't try and find the end;
+				// there is not much text after the bundle dump anyway, and
+				// attempts to find it can end up truncating the matched block
+				// and hiding the useful information.
+				Pattern pattern = Pattern.compile("^Id\\s+Levl\\s+State.*", Pattern.DOTALL | Pattern.MULTILINE);
+				Matcher match = pattern.matcher(out);
+				if (!match.find()) {
+					throw new AssertionError("Couldn't find report block in output:\n" + out);
+				}
+				final List<String> report = Strings.splitLines(match.group());
 
-			String[] expected = {
-				"org.apache.felix.scr", "org.apache.servicemix.bundles.junit", "org.assertj.core", "demo"
-			};
+				List<String> expected = Arrays.asList("System Bundle", "org.apache.felix.scr",
+					"org.apache.servicemix.bundles.junit", "assertj-core", "demo");
 
-			l.launch();
-
-			// High-level test; cuts down our search space for subsequent tests
-			// which makes the output more readable in the event of a subsequent
-			// assertion failure.
-			Matcher match = Pattern.compile("^Id\\s+\\s+Levl\\s+State.*?(?=^#)", Pattern.DOTALL | Pattern.MULTILINE)
-				.matcher(err);
-			assertThat(match.find()).as("report block")
-				.isTrue();
-			final String report = match.group();
-
-			softly.assertThat(report)
-				.as("System Bundle")
-				.containsPattern("\\n0\\s+\\d+\\s*ACTIV\\s+[<][>]\\s+System Bundle");
-
-			final DateTimeFormatter f = DateTimeFormatter.ofPattern("YYYYMMddHHmm")
-				.withZone(ZoneOffset.UTC);
-
-			for (String bundle : expected) {
-				final Path file = Paths.get(r.process("${repo;" + bundle + ";latest}"));
-				final String lastModified = f.format(java.nio.file.Files.getLastModifiedTime(file)
-					.toInstant());
-
-				softly.assertThat(report)
-					.as(bundle)
-					.containsPattern("ACTIV\\s+" + lastModified + "\\s+.*?\\Q" + file.getFileName() + "\\E");
+				for (String bundle : expected) {
+					Pattern bundlePattern = Pattern.compile("ACTIV.*\\Q" + bundle + "\\E");
+					softly.assertThat(report)
+						.as(bundle)
+						.anyMatch(line -> bundlePattern.matcher(line)
+							.find());
+				}
 			}
 		}
 	}
@@ -854,7 +1047,7 @@ public class AlsoLauncherTest {
 	@Test
 	public void testTimeoutActivator() throws Exception {
 		try (ProjectLauncher l = project.getProjectLauncher()) {
-			l.setTimeout(100, TimeUnit.MILLISECONDS);
+			l.setTimeout(100L, TimeUnit.MILLISECONDS);
 			l.setTrace(false);
 			assertThat(l.launch()).isEqualTo(ProjectLauncher.TIMEDOUT);
 		}
@@ -864,7 +1057,7 @@ public class AlsoLauncherTest {
 	@Test
 	public void testTimeout() throws Exception {
 		try (ProjectLauncher l = project.getProjectLauncher()) {
-			l.setTimeout(100, TimeUnit.MILLISECONDS);
+			l.setTimeout(100L, TimeUnit.MILLISECONDS);
 			l.setTrace(false);
 			l.getRunProperties()
 				.put("test.cmd", "timeout");
@@ -897,6 +1090,11 @@ public class AlsoLauncherTest {
 		assertExitCode("main.thread.callableinvalidtype", 0);
 	}
 
+	@Test
+	public void testMainThreadCallableDontStopOnFirstCall() throws Exception {
+		assertExitCode("main.thread.callable.two.runs", 0);
+	}
+
 	@Test
 	public void testMainThreadCallable() throws Exception {
 		assertExitCode("main.thread.callable", 42);
@@ -909,7 +1107,7 @@ public class AlsoLauncherTest {
 
 	private void assertExitCode(String cmd, int rv) throws Exception {
 		try (ProjectLauncher l = project.getProjectLauncher()) {
-			l.setTimeout(25000, TimeUnit.MILLISECONDS);
+			l.setTimeout(25000L, TimeUnit.MILLISECONDS);
 			l.setTrace(true);
 			l.getRunProperties()
 				.put("test.cmd", cmd);
@@ -926,7 +1124,7 @@ public class AlsoLauncherTest {
 			runbundles + "," + new File("jar/mandatorynoversion.jar;version=file").getAbsolutePath());
 		ProjectTester tester = project.getProjectTester();
 		try (ProjectLauncher l = tester.getProjectLauncher()) {
-			l.setTimeout(25000, TimeUnit.MILLISECONDS);
+			l.setTimeout(25000L, TimeUnit.MILLISECONDS);
 			l.setTrace(true);
 			AtomicBoolean reported = new AtomicBoolean(false);
 			tester.registerForNotifications((a, b) -> {
@@ -956,7 +1154,7 @@ public class AlsoLauncherTest {
 						error.set(notification);
 					}
 				});
-				l.setTimeout(5000, TimeUnit.MILLISECONDS);
+				l.setTimeout(5000L, TimeUnit.MILLISECONDS);
 				l.setTrace(true);
 				l.launch();
 
@@ -969,6 +1167,8 @@ public class AlsoLauncherTest {
 	public void testOlderLauncherOnRunpath() throws Exception {
 		try (Run run = new Run(project.getWorkspace(), project.getFile("old-launcher.bndrun"))) {
 			run.setProperty(Constants.RUNTRACE, "true");
+			// old launcher has a bug which tries to use invalid "MD-5" digest
+			run.setProperty(Constants.DIGESTS, "SHA-1");
 
 			File file = new File(testDir, "packaged.jar");
 			try (Jar pack = run.pack(null)) {
