@@ -1,6 +1,10 @@
 package aQute.bnd.comm.tests;
 
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
 import java.io.IOException;
 import java.math.BigInteger;
@@ -17,6 +21,7 @@ import java.util.Date;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.IntSupplier;
 
 import javax.net.ssl.SSLEngine;
 import javax.net.ssl.SSLException;
@@ -29,6 +34,8 @@ import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
 import org.bouncycastle.cert.jcajce.JcaX509v3CertificateBuilder;
 import org.bouncycastle.operator.ContentSigner;
 import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Test;
 import org.littleshoot.proxy.HttpFilters;
 import org.littleshoot.proxy.HttpFiltersAdapter;
 import org.littleshoot.proxy.HttpFiltersSource;
@@ -45,6 +52,7 @@ import aQute.bnd.http.HttpClient;
 import aQute.bnd.osgi.Processor;
 import aQute.bnd.service.url.State;
 import aQute.bnd.service.url.TaggedData;
+import aQute.bnd.test.net.EphemeralPort;
 import aQute.http.testservers.HttpTestServer;
 import aQute.http.testservers.Httpbin;
 import aQute.lib.io.IO;
@@ -58,7 +66,6 @@ import io.netty.handler.codec.http.HttpResponse;
 import io.netty.handler.codec.http.HttpResponseStatus;
 import io.netty.handler.codec.http.HttpVersion;
 import io.netty.handler.ssl.SslContextBuilder;
-import junit.framework.TestCase;
 import sockslib.common.AuthenticationException;
 import sockslib.common.Credentials;
 import sockslib.common.methods.UsernamePasswordMethod;
@@ -78,7 +85,7 @@ import sockslib.server.msg.CommandMessage;
  * Different combinations are tried out with a secure and unsecure server. See
  * https://github.com/fengyouchao/sockslib for socks
  */
-public class HttpClientProxyTest extends TestCase {
+public class HttpClientProxyTest {
 
 	private final class ProxyCheckingFilter implements HttpFiltersSource {
 
@@ -125,18 +132,21 @@ public class HttpClientProxyTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testPromiscuousProxyWithNoUser() throws Exception {
 		createPromiscuousHttpProxy();
 		createUnsecureServer();
 		assertHttpProxy(null, false, 200);
 	}
 
+	@Test
 	public void testPromiscuousProxyWithBadUser() throws Exception {
 		createPromiscuousHttpProxy();
 		createUnsecureServer();
 		assertHttpProxy("bad", false, 200);
 	}
 
+	@Test
 	public void testAuthenticatingProxyWithGoodUser() throws Exception {
 		createAuthenticationHttpProxy();
 		createUnsecureServer();
@@ -144,28 +154,33 @@ public class HttpClientProxyTest extends TestCase {
 		assertHttpProxy("good", true, 200);
 	}
 
+	@Test
 	public void testPromiscuousProxyWithGoodUser() throws Exception {
 		createPromiscuousHttpProxy();
 		createUnsecureServer();
 		assertHttpProxy("good", false, 200);
 	}
 
+	@Test
 	public void testNoProxyUnsecure() throws Exception {
 		createUnsecureServer();
 		assertHttpProxy("good", Type.HTTP.name(), false, false, 200);
 	}
 
+	@Test
 	public void testNoProxySecure() throws Exception {
 		createUnsecureServer();
 		assertHttpProxy("good", Type.HTTP.name(), false, false, 200);
 	}
 
+	@Test
 	public void testPromiscuousProxyWithGoodUserSecure() throws Exception {
 		createSecureServer();
 		createSecurePromiscuousHttpProxy();
 		assertHttpProxy("good", "HTTPS", true, false, 200);
 	}
 
+	@Test
 	public void testAuthenticatingProxyNoUser() throws Exception {
 		try {
 			createAuthenticationHttpProxy();
@@ -178,6 +193,7 @@ public class HttpClientProxyTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testAuthenticatingProxyBadUser() throws Exception {
 		try {
 			createAuthenticationHttpProxy();
@@ -190,24 +206,28 @@ public class HttpClientProxyTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testSecureSocksAuthenticatingWithGoodUserSecure() throws Exception {
 		createSecureSocks5();
 		createSecureServer();
 		assertThat(assertSocks5Proxy("good", true)).isBetween(200, 299);
 	}
 
+	@Test
 	public void testSecureSocksAuthenticatingWithBadUserSecure() throws Exception {
 		createSecureSocks5();
 		createSecureServer();
 		assertThat(assertSocks5Proxy("bad", true)).isGreaterThanOrEqualTo(500);
 	}
 
+	@Test
 	public void testSecureSocksAuthenticatingWithGoodUser() throws Exception {
 		createSecureSocks5();
 		createUnsecureServer();
 		assertThat(assertSocks5Proxy("good", true)).isBetween(200, 299);
 	}
 
+	@Test
 	public void testSecureSocksAuthenticatingWithBadUser() throws Exception {
 		createSecureSocks5();
 		createUnsecureServer();
@@ -223,8 +243,8 @@ public class HttpClientProxyTest extends TestCase {
 	private AtomicBoolean				proxyCalled				= new AtomicBoolean();
 	private HttpTestServer				httpTestServer;
 	private SocksProxyServer			socks5Proxy;
-	private static final AtomicInteger	httpProxyPort			= new AtomicInteger(IO.isWindows() ? 52080 : 2080);
-	private static final AtomicInteger	socksProxyPort			= new AtomicInteger(IO.isWindows() ? 53080 : 3080);
+
+	static final IntSupplier			portSupplier			= EphemeralPort.AUTOMATIC;
 
 	private AtomicReference<Throwable>	exception				= new AtomicReference<>();
 	private AtomicInteger				created					= new AtomicInteger();
@@ -232,102 +252,136 @@ public class HttpClientProxyTest extends TestCase {
 	// we use different ports because the servers seem to linger
 
 	void createAuthenticationHttpProxy() {
-		HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
-			.withPort(httpProxyPort.incrementAndGet())
-			.withProxyAuthenticator(new ProxyAuthenticator() {
+		int retry = 5;
+		while (true) {
+			final int port = portSupplier.getAsInt();
+			HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
+				.withPort(port)
+				.withProxyAuthenticator(new ProxyAuthenticator() {
 
-				@Override
-				public boolean authenticate(String user, String password) {
-					System.out.println("Authenticating " + user + " : " + password);
-					authenticationCalled.set(true);
-					return "proxyuser".equals(user) && "good".equals(password);
+					@Override
+					public boolean authenticate(String user, String password) {
+						System.out.println("Authenticating " + user + " : " + password);
+						authenticationCalled.set(true);
+						return "proxyuser".equals(user) && "good".equals(password);
 
+					}
+
+					@Override
+					public String getRealm() {
+						return null;
+					}
+				})
+				.withFiltersSource(new ProxyCheckingFilter(false));
+			try {
+				httpProxy = bootstrap.start();
+				break;
+			} catch (Exception e) {
+				if (--retry <= 0) {
+					throw e;
 				}
-
-				@Override
-				public String getRealm() {
-					return null;
-				}
-			})
-			.withFiltersSource(new ProxyCheckingFilter(false));
-		httpProxy = bootstrap.start();
+			}
+		}
 	}
 
 	void createPromiscuousHttpProxy() {
-		HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
-			.withPort(httpProxyPort.incrementAndGet())
-			.withFiltersSource(new ProxyCheckingFilter(false));
-		httpProxy = bootstrap.start();
+		int retry = 5;
+		while (true) {
+			final int port = portSupplier.getAsInt();
+			HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
+				.withPort(port)
+				.withFiltersSource(new ProxyCheckingFilter(false));
+			try {
+				httpProxy = bootstrap.start();
+				break;
+			} catch (Exception e) {
+				if (--retry <= 0) {
+					throw e;
+				}
+			}
+		}
 	}
 
 	void createSecurePromiscuousHttpProxy() {
-		HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
-			.withPort(httpProxyPort.incrementAndGet())
-			.withManInTheMiddle(new MitmManager() {
+		int retry = 5;
+		while (true) {
+			final int port = portSupplier.getAsInt();
+			HttpProxyServerBootstrap bootstrap = DefaultHttpProxyServer.bootstrap()
+				.withPort(port)
+				.withManInTheMiddle(new MitmManager() {
 
-				SecureRandom random = new SecureRandom();
+					SecureRandom random = new SecureRandom();
 
-				@Override
-				public SSLEngine serverSslEngine(String arg0, int arg1) {
-					return serverSslEngine();
-				}
-
-				@Override
-				public SSLEngine serverSslEngine() {
-					try {
-						return SslContextBuilder.forClient()
-							.trustManager(httpTestServer.getCertificateChain())
-							.build()
-							.newEngine(UnpooledByteBufAllocator.DEFAULT);
-					} catch (SSLException e) {
-						throw new RuntimeException(e);
+					@Override
+					public SSLEngine serverSslEngine(String arg0, int arg1) {
+						return serverSslEngine();
 					}
-				}
 
-				@SuppressWarnings("restriction")
-				private X509Certificate[] createSelfSignedCertifcate(KeyPair keyPair) throws Exception {
-					X500NameBuilder nameBuilder = new X500NameBuilder(BCStyle.INSTANCE);
-					nameBuilder.addRDN(BCStyle.CN, "localhost");
-
-					Date notBefore = new Date();
-					Date notAfter = new Date(System.currentTimeMillis() + 24 * 3 * 60 * 60 * 1000);
-
-					BigInteger serialNumber = new BigInteger(128, random);
-
-					X509v3CertificateBuilder certificateBuilder = new JcaX509v3CertificateBuilder(nameBuilder.build(),
-						serialNumber, notBefore, notAfter, nameBuilder.build(), keyPair.getPublic());
-					ContentSigner contentSigner = new JcaContentSignerBuilder("SHA256WithRSAEncryption")
-						.build(keyPair.getPrivate());
-					X509Certificate certificate = new JcaX509CertificateConverter()
-						.getCertificate(certificateBuilder.build(contentSigner));
-					return new X509Certificate[] {
-						certificate
-					};
-				}
-
-				private KeyPair createKey() throws NoSuchAlgorithmException {
-					KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
-					keyGen.initialize(2048, random);
-					KeyPair pair = keyGen.generateKeyPair();
-					return pair;
-				}
-
-				@Override
-				public SSLEngine clientSslEngineFor(HttpRequest arg0, SSLSession arg1) {
-					try {
-						KeyPair keyPair = createKey();
-						X509Certificate[] selfSignedCertifcate = createSelfSignedCertifcate(keyPair);
-
-						return SslContextBuilder.forServer(keyPair.getPrivate(), selfSignedCertifcate)
-							.build()
-							.newEngine(UnpooledByteBufAllocator.DEFAULT);
-					} catch (Exception e) {
-						throw new RuntimeException(e);
+					@Override
+					public SSLEngine serverSslEngine() {
+						try {
+							return SslContextBuilder.forClient()
+								.trustManager(httpTestServer.getCertificateChain())
+								.build()
+								.newEngine(UnpooledByteBufAllocator.DEFAULT);
+						} catch (SSLException e) {
+							throw new RuntimeException(e);
+						}
 					}
+
+					@SuppressWarnings("restriction")
+					private X509Certificate[] createSelfSignedCertifcate(KeyPair keyPair) throws Exception {
+						X500NameBuilder nameBuilder = new X500NameBuilder(BCStyle.INSTANCE);
+						nameBuilder.addRDN(BCStyle.CN, "localhost");
+
+						Date notBefore = new Date();
+						Date notAfter = new Date(System.currentTimeMillis() + 24 * 3 * 60 * 60 * 1000);
+
+						BigInteger serialNumber = new BigInteger(128, random);
+
+						X509v3CertificateBuilder certificateBuilder = new JcaX509v3CertificateBuilder(
+							nameBuilder.build(), serialNumber, notBefore, notAfter, nameBuilder.build(),
+							keyPair.getPublic());
+						ContentSigner contentSigner = new JcaContentSignerBuilder("SHA256WithRSAEncryption")
+							.build(keyPair.getPrivate());
+						X509Certificate certificate = new JcaX509CertificateConverter()
+							.getCertificate(certificateBuilder.build(contentSigner));
+						return new X509Certificate[] {
+							certificate
+						};
+					}
+
+					private KeyPair createKey() throws NoSuchAlgorithmException {
+						KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
+						keyGen.initialize(2048, random);
+						KeyPair pair = keyGen.generateKeyPair();
+						return pair;
+					}
+
+					@Override
+					public SSLEngine clientSslEngineFor(HttpRequest arg0, SSLSession arg1) {
+						try {
+							KeyPair keyPair = createKey();
+							X509Certificate[] selfSignedCertifcate = createSelfSignedCertifcate(keyPair);
+
+							return SslContextBuilder.forServer(keyPair.getPrivate(), selfSignedCertifcate)
+								.build()
+								.newEngine(UnpooledByteBufAllocator.DEFAULT);
+						} catch (Exception e) {
+							throw new RuntimeException(e);
+						}
+					}
+				})
+				.withFiltersSource(new ProxyCheckingFilter(true));
+			try {
+				httpProxy = bootstrap.start();
+				break;
+			} catch (Exception e) {
+				if (--retry <= 0) {
+					throw e;
 				}
-			})
-			.withFiltersSource(new ProxyCheckingFilter(true));
-		httpProxy = bootstrap.start();
+			}
+		}
 	}
 
 	void createSecureServer() throws Exception {
@@ -345,54 +399,71 @@ public class HttpClientProxyTest extends TestCase {
 	}
 
 	private void createSecureSocks5() throws IOException, InterruptedException {
-		UserManager userManager = new MemoryBasedUserManager();
-		userManager.create(new User("proxyuser", "good"));
-		SocksServerBuilder builder = SocksServerBuilder.newSocks5ServerBuilder()
-			.setBindPort(socksProxyPort.incrementAndGet())
-			.addSocksMethods(new UsernamePasswordMethod(new UsernamePasswordAuthenticator(userManager) {
-				@Override
-				public void doAuthenticate(Credentials arg0, Session arg1) throws AuthenticationException {
-					System.out.println("Authenticating"); // does not get
-															// called?
-					super.doAuthenticate(arg0, arg1);
-					authenticationCalled.set(true);
+		int retry = 5;
+		IOException fail = null;
+		while (true) {
+			final int socksProxyPort = portSupplier.getAsInt();
+			UserManager userManager = new MemoryBasedUserManager();
+			userManager.create(new User("proxyuser", "good"));
+			SocksServerBuilder builder = SocksServerBuilder.newSocks5ServerBuilder()
+				.setBindPort(socksProxyPort)
+				.addSocksMethods(new UsernamePasswordMethod(new UsernamePasswordAuthenticator(userManager) {
+					@Override
+					public void doAuthenticate(Credentials arg0, Session arg1) throws AuthenticationException {
+						System.out.println("Authenticating"); // does not get
+						// called?
+						super.doAuthenticate(arg0, arg1);
+						authenticationCalled.set(true);
 
+					}
+				}));
+			socks5Proxy = builder.build();
+
+			socks5Proxy.getSessionManager()
+				.addSessionListener("abc", new SessionListener() {
+
+					@Override
+					public void onException(Session arg0, Exception arg1) {
+						System.err.println("Exception " + arg0 + " " + arg1);
+						arg1.printStackTrace();
+						exception.set(arg1);
+					}
+
+					@Override
+					public void onCommand(Session arg0, CommandMessage arg1) throws CloseSessionException {
+						proxyCalled.set(true);
+						System.err.println("Command " + arg0 + " " + arg1);
+					}
+
+					@Override
+					public void onClose(Session arg0) {
+						System.err.println("Close " + arg0);
+					}
+
+					@Override
+					public void onCreate(Session arg0) throws CloseSessionException {
+						System.err.println("Create " + arg0);
+						created.incrementAndGet();
+					}
+				});
+
+			try {
+				socks5Proxy.start();
+				break;
+			} catch (Exception e) {
+				IOException x = new IOException("Failed to start Socks Proxy Server on port " + socksProxyPort, e);
+				if (fail != null) {
+					x.addSuppressed(fail);
 				}
-			}));
-		socks5Proxy = builder.build();
-
-		socks5Proxy.getSessionManager()
-			.addSessionListener("abc", new SessionListener() {
-
-				@Override
-				public void onException(Session arg0, Exception arg1) {
-					System.err.println("Exception " + arg0 + " " + arg1);
-					arg1.printStackTrace();
-					exception.set(arg1);
+				if (--retry <= 0) {
+					throw x;
 				}
-
-				@Override
-				public void onCommand(Session arg0, CommandMessage arg1) throws CloseSessionException {
-					proxyCalled.set(true);
-					System.err.println("Command " + arg0 + " " + arg1);
-				}
-
-				@Override
-				public void onClose(Session arg0) {
-					System.err.println("Close " + arg0);
-				}
-
-				@Override
-				public void onCreate(Session arg0) throws CloseSessionException {
-					System.err.println("Create " + arg0);
-					created.incrementAndGet();
-				}
-			});
-
-		socks5Proxy.start();
+				fail = x;
+			}
+		}
 	}
 
-	@Override
+	@AfterEach
 	protected void tearDown() throws Exception {
 		if (httpProxy != null) {
 			httpProxy.abort();
@@ -405,8 +476,6 @@ public class HttpClientProxyTest extends TestCase {
 		if (socks5Proxy != null) {
 			socks5Proxy.shutdown();
 		}
-
-		super.tearDown();
 	}
 
 	void assertHttpProxy(String password, boolean authenticationCalled, int responseCode)
