@@ -2,18 +2,31 @@ package aQute.bnd.comm.tests;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
 import java.io.File;
 import java.net.HttpURLConnection;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
+import java.net.URLConnection;
 import java.nio.ByteBuffer;
 import java.security.cert.X509Certificate;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.opentest4j.AssertionFailedError;
 import org.osgi.util.promise.Promise;
 
 import aQute.bnd.connection.settings.ConnectionSettings;
@@ -25,23 +38,22 @@ import aQute.bnd.service.progress.ProgressPlugin;
 import aQute.bnd.service.progress.ProgressPlugin.Task;
 import aQute.bnd.service.url.State;
 import aQute.bnd.service.url.TaggedData;
+import aQute.bnd.service.url.URLConnectionHandler;
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.bnd.url.HttpsVerification;
 import aQute.http.testservers.HttpTestServer.Config;
 import aQute.http.testservers.Httpbin;
 import aQute.lib.io.IO;
 import aQute.lib.strings.Strings;
-import junit.framework.TestCase;
 
-public class HttpClientTest extends TestCase {
+public class HttpClientTest {
 	private TestServer	httpServer;
 	private Httpbin		httpsServer;
-	private File		tmp;
 
-	@Override
+	@AfterEach
 	protected void tearDown() throws Exception {
 		IO.close(httpServer);
 		IO.close(httpsServer);
-		super.tearDown();
 	}
 
 	public static class TestServer extends Httpbin {
@@ -73,6 +85,19 @@ public class HttpClientTest extends TestCase {
 			}
 		}
 
+		AtomicInteger n = new AtomicInteger();
+
+		public void _solitary(Request rq, Response rsp, int max) throws Exception {
+			System.out.println("in solitary");
+			int x = n.incrementAndGet();
+			if (x > max) {
+				rsp.code = 400;
+				return;
+			}
+			Thread.sleep(200);
+			x = n.decrementAndGet();
+		}
+
 		volatile boolean firstTimeout = true;
 
 		public void _readtimeout(Request rq, Response rsp, int stage) throws InterruptedException {
@@ -116,13 +141,8 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	private String getTestName() {
-		return getClass().getName() + "/" + getName();
-	}
-
-	@Override
+	@BeforeEach
 	protected void setUp() throws Exception {
-		super.setUp();
 		Config config = new Config();
 		config.https = false;
 		httpServer = new TestServer(config);
@@ -133,12 +153,75 @@ public class HttpClientTest extends TestCase {
 		httpsServer = new Httpbin(configs);
 		httpsServer.start();
 
-		tmp = IO.getFile("generated/tmp/test/" + getTestName());
-		IO.delete(tmp);
-		tmp.mkdirs();
 		httpServer.second = false;
 	}
 
+	@Test
+	public void testLimitConnections() throws Exception {
+		try (Processor p = new Processor()) {
+			try (HttpClient client = new HttpClient()) {
+				AtomicInteger max = new AtomicInteger(1);
+				client.addURLConnectionHandler(new URLConnectionHandler() {
+					@Override
+					public boolean matches(URL url) {
+						return true;
+					}
+
+					@Override
+					public void handle(URLConnection connection) throws Exception {}
+
+					@Override
+					public int maxConcurrentConnections() {
+						return max.get();
+					}
+				});
+
+				testParallel(client, max.get());
+				max.set(3);
+				testParallel(client, max.get());
+				try {
+					max.set(0);
+					testParallel(client, 1);
+					fail("expected a failure");
+				} catch (AssertionFailedError e) {
+					// ok
+				}
+
+			}
+		}
+	}
+
+	private void testParallel(HttpClient client, int max) {
+		List<CompletableFuture<TaggedData>> fs = new ArrayList<>();
+		for (int n = 0; n < 10; n++) {
+			CompletableFuture<TaggedData> f = CompletableFuture.supplyAsync(() -> {
+				try {
+					TaggedData tag = client.build()
+						.timeout(10000)
+						.asTag()
+						.go(httpServer.getBaseURI("solitary/" + max));
+					System.out.println("response "+tag.getResponseCode());
+					return tag;
+				} catch (Exception e) {
+					throw new RuntimeException();
+				}
+			});
+			fs.add(f);
+		}
+
+		fs.forEach(tag -> {
+			try {
+				TaggedData d = tag.get();
+				assertThat(d).isNotNull();
+				TaggedData taggedData = tag.get();
+				assertThat(taggedData.getResponseCode()).isEqualTo(200);
+			} catch (InterruptedException | ExecutionException e) {
+				throw new RuntimeException();
+			}
+		});
+	}
+
+	@Test
 	public void testReadTimeOutButSucceed() throws Exception {
 		try (Processor p = new Processor()) {
 			try (HttpClient client = new HttpClient()) {
@@ -155,7 +238,27 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testNoRetriesOnPut() throws Exception {
+		try (Processor p = new Processor()) {
+			try (HttpClient client = new HttpClient()) {
+				client.setReporter(p);
+
+				TaggedData tag = client.build()
+					.timeout(1000)
+					.verb("PUT")
+					.idemPotent(false)
+					.retries(2)
+					.asTag()
+					.go(httpServer.getBaseURI("readtimeout/1"));
+				System.out.println(tag);
+				assertThat(tag.getResponseCode()).isEqualTo(HttpURLConnection.HTTP_GATEWAY_TIMEOUT);
+			}
+		}
+	}
+
+	@Test
+	public void testRetriesOnPut() throws Exception {
 		try (Processor p = new Processor()) {
 			try (HttpClient client = new HttpClient()) {
 				client.setReporter(p);
@@ -167,11 +270,12 @@ public class HttpClientTest extends TestCase {
 					.asTag()
 					.go(httpServer.getBaseURI("readtimeout/1"));
 				System.out.println(tag);
-				assertThat(tag.getResponseCode()).isEqualTo(HttpURLConnection.HTTP_GATEWAY_TIMEOUT);
+				assertThat(tag.getResponseCode()).isEqualTo(HttpURLConnection.HTTP_OK);
 			}
 		}
 	}
 
+	@Test
 	public void testReadTimeOutButFail() throws Exception {
 		try (Processor p = new Processor()) {
 			try (HttpClient client = new HttpClient()) {
@@ -188,6 +292,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testReadFailRetrySucceeds() throws Exception {
 		try (Processor p = new Processor()) {
 			try (HttpClient client = new HttpClient()) {
@@ -203,6 +308,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testReadFailRetryFails() throws Exception {
 		try (Processor p = new Processor()) {
 			try (HttpClient client = new HttpClient()) {
@@ -219,7 +325,9 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	public void testHttpsVerification() throws Exception {
+	@Test
+	public void testHttpsVerification(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (Processor p = new Processor()) {
 			p.setProperty("-connection-settings", "server;id=\"" + httpsServer.getBaseURI() + "\";verify=" + true
 				+ ";trust=\"" + Strings.join(httpsServer.getTrustedCertificateFiles(tmp)) + "\"");
@@ -270,11 +378,14 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	public void testHostnameVerification() throws Exception {
+	@Test
+	public void testHostnameVerification(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (Processor p = new Processor()) {
 
 			//
-			// This should fail as the CN of the service cert does not equal the hostname ("localhost")
+			// This should fail as the CN of the service cert does not equal the
+			// hostname ("localhost")
 			//
 
 			Config configs = new Config();
@@ -282,12 +393,12 @@ public class HttpClientTest extends TestCase {
 			try (Httpbin extraServer = new Httpbin(configs, "somehost")) {
 				extraServer.start();
 
-                                p.setProperty("-connection-settings", "server;id=\"" + extraServer.getBaseURI() + "\";verify=" + true
-				        + ";trust=\"" + Strings.join(extraServer.getTrustedCertificateFiles(tmp)) + "\"");
-			        HttpClient client = new HttpClient();
-			        client.setReporter(p);
-			        ConnectionSettings cs = new ConnectionSettings(p, client);
-			        cs.readSettings();
+				p.setProperty("-connection-settings", "server;id=\"" + extraServer.getBaseURI() + "\";verify=" + true
+					+ ";trust=\"" + Strings.join(extraServer.getTrustedCertificateFiles(tmp)) + "\"");
+				HttpClient client = new HttpClient();
+				client.setReporter(p);
+				ConnectionSettings cs = new ConnectionSettings(p, client);
+				cs.readSettings();
 
 				TaggedData go3 = client.build()
 					.retries(0)
@@ -299,11 +410,14 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	public void testHostnameVerificationDisabled() throws Exception {
+	@Test
+	public void testHostnameVerificationDisabled(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (Processor p = new Processor()) {
 
 			//
-			// This should pass even though the CN of the service cert does not equal the hostname ("localhost"),
+			// This should pass even though the CN of the service cert does not
+			// equal the hostname ("localhost"),
 			// because verification is disabled
 			//
 
@@ -312,12 +426,12 @@ public class HttpClientTest extends TestCase {
 			try (Httpbin extraServer = new Httpbin(configs, "somehost")) {
 				extraServer.start();
 
-                                p.setProperty("-connection-settings", "server;id=\"" + extraServer.getBaseURI() + "\";verify=" + false
-				        + ";trust=\"" + Strings.join(extraServer.getTrustedCertificateFiles(tmp)) + "\"");
-			        HttpClient client = new HttpClient();
-			        client.setReporter(p);
-			        ConnectionSettings cs = new ConnectionSettings(p, client);
-			        cs.readSettings();
+				p.setProperty("-connection-settings", "server;id=\"" + extraServer.getBaseURI() + "\";verify=" + false
+					+ ";trust=\"" + Strings.join(extraServer.getTrustedCertificateFiles(tmp)) + "\"");
+				HttpClient client = new HttpClient();
+				client.setReporter(p);
+				ConnectionSettings cs = new ConnectionSettings(p, client);
+				cs.readSettings();
 
 				TaggedData go3 = client.build()
 					.retries(0)
@@ -329,6 +443,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testTimeout() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			try {
@@ -347,7 +462,9 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	public void testClearCacheOn404() throws Exception {
+	@Test
+	public void testClearCacheOn404(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			hc.setCache(tmp);
 			URLCache cache = hc.cache();
@@ -378,6 +495,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testCancel() throws Exception {
 		final long deadline = System.currentTimeMillis() + 1000L;
 
@@ -420,7 +538,9 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
-	public void testCachingMultipleFetch() throws Exception {
+	@Test
+	public void testCachingMultipleFetch(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			hc.setCache(tmp);
 
@@ -457,6 +577,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testFetch() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			String text = hc.build()
@@ -467,6 +588,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testURLResource() throws Exception {
 		try (HttpClient hc = new HttpClient()) {
 			URL url = httpServer.getBaseURI("get")
@@ -480,6 +602,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testRedirect() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData tag = hc.build()
@@ -490,6 +613,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testRedirectRelative() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData tag = hc.build()
@@ -500,6 +624,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testRedirectTooMany() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData tag = hc.build()
@@ -510,6 +635,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testThatHttpWithUnverifiedNameGeneratesError() throws Exception {
 		Processor p = new Processor();
 		try (HttpClient hc = new HttpClient();) {
@@ -526,6 +652,7 @@ public class HttpClientTest extends TestCase {
 		assertTrue(p.check());
 	}
 
+	@Test
 	public void testThatHttpWithUnverifiedNameButMatchingHandlerIsOk() throws Exception {
 		Processor p = new Processor();
 		try (HttpClient hc = new HttpClient();) {
@@ -546,6 +673,7 @@ public class HttpClientTest extends TestCase {
 		assertTrue(p.check());
 	}
 
+	@Test
 	public void testInvalidTrustAnchors() throws Exception {
 		Processor p = new Processor();
 		try (HttpClient hc = new HttpClient();) {
@@ -572,6 +700,7 @@ public class HttpClientTest extends TestCase {
 		assertTrue(p.check());
 	}
 
+	@Test
 	public void testRedirectURL() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			HttpsVerification httpsVerification = new HttpsVerification(httpsServer.getCertificateChain(), false,
@@ -591,6 +720,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testETag() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -601,6 +731,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testNotModifiedEtag() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -613,6 +744,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testModifiedWithEtag() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -625,6 +757,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testNotModifiedSince() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -638,6 +771,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testNotModifiedSinceAtSameTime() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -651,6 +785,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testModifiedSince() throws Exception {
 		try (HttpClient hc = new HttpClient();) {
 			TaggedData data = hc.build()
@@ -672,6 +807,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testMultipleProgressPlugins() throws Exception {
 		final long deadline = System.currentTimeMillis() + 1000L;
 
@@ -724,6 +860,7 @@ public class HttpClientTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testPut() throws URISyntaxException, Exception {
 		try (Processor p = new Processor();) {
 			final AtomicBoolean done = new AtomicBoolean();
