@@ -1,30 +1,25 @@
 package test;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+
 import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.junit.jupiter.api.Test;
+
 import aQute.bnd.build.Container;
 import aQute.bnd.header.Attrs;
 import aQute.bnd.osgi.Builder;
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.lib.io.IO;
-import junit.framework.TestCase;
 
-public class ContainerTest extends TestCase {
-	File tmp;
+public class ContainerTest {
 
-	private String getTestName() {
-		return getClass().getName() + "/" + getName();
-	}
-
-	@Override
-	public void setUp() {
-		tmp = IO.getFile("generated/tmp/test/" + getTestName());
-		IO.delete(tmp);
-		tmp.mkdirs();
-	}
-
-	public void testBundleClasspath() throws Exception {
+	@Test
+	public void testBundleClasspath(@InjectTemporaryDirectory
+	File tmp) throws Exception {
 		try (Builder b = new Builder();) {
 			b.addClasspath(new File("bin_test"));
 			b.setBundleClasspath(".,osgi.jar");
