@@ -4,5 +4,4 @@ import org.apiguardian.api.API;
 import org.apiguardian.api.API.Status;
 
 @API(status = Status.INTERNAL)
-public interface Foo2 {
-}
+public interface Foo2 {}
