@@ -239,7 +239,7 @@ public class NanoHTTPD {
 				port = Integer.parseInt(args[i + 1]);
 			else if (args[i].equalsIgnoreCase("-d"))
 				wwwroot = new File(args[i + 1]).getAbsoluteFile();
-			else if (args[i].toLowerCase()
+			else if (args[i].toLowerCase(Locale.ROOT)
 				.endsWith("licence")) {
 				myOut.println(LICENCE + "\n");
 				break;
@@ -302,7 +302,7 @@ public class NanoHTTPD {
 				String method = pre.getProperty("method");
 				String uri = pre.getProperty("uri");
 
-				long size = 0x7FFFFFFFFFFFFFFFl;
+				long size = 0x7FFFFFFFFFFFFFFFL;
 				String contentLength = header.getProperty("content-length");
 				if (contentLength != null) {
 					try {
@@ -339,7 +339,7 @@ public class NanoHTTPD {
 				// expect the first byte of the body at the next read.
 				if (splitbyte < rlen)
 					size -= rlen - splitbyte + 1;
-				else if (!sbfound || size == 0x7FFFFFFFFFFFFFFFl)
+				else if (!sbfound || size == 0x7FFFFFFFFFFFFFFFL)
 					size = 0;
 
 				// Now read all the body and write it to f
@@ -460,7 +460,7 @@ public class NanoHTTPD {
 						if (p >= 0)
 							header.put(line.substring(0, p)
 								.trim()
-								.toLowerCase(),
+								.toLowerCase(Locale.ROOT),
 								line.substring(p + 1)
 									.trim());
 						line = in.readLine();
@@ -496,7 +496,7 @@ public class NanoHTTPD {
 						if (p != -1)
 							item.put(mpline.substring(0, p)
 								.trim()
-								.toLowerCase(),
+								.toLowerCase(Locale.ROOT),
 								mpline.substring(p + 1)
 									.trim());
 						mpline = in.readLine();
@@ -515,7 +515,7 @@ public class NanoHTTPD {
 							if (p != -1)
 								disposition.put(token.substring(0, p)
 									.trim()
-									.toLowerCase(),
+									.toLowerCase(Locale.ROOT),
 									token.substring(p + 1)
 										.trim());
 						}
@@ -816,7 +816,6 @@ public class NanoHTTPD {
 					f = IO.getFile(homeDir, uri + "/index.htm");
 				// No index file, list the directory if it is readable
 				else if (allowDirectoryListing && f.canRead()) {
-					String[] files = f.list();
 					String msg = "<html><body><h1>Directory " + uri + "</h1><br/>";
 
 					if (uri.length() > 1) {
@@ -826,34 +825,32 @@ public class NanoHTTPD {
 							msg += "<b><a href=\"" + uri.substring(0, slash + 1) + "\">..</a></b><br/>";
 					}
 
-					if (files != null) {
-						for (int i = 0; i < files.length; ++i) {
-							File curFile = new File(f, files[i]);
-							boolean dir = curFile.isDirectory();
-							if (dir) {
-								msg += "<b>";
-								files[i] += "/";
-							}
-
-							msg += "<a href=\"" + encodeUri(uri + files[i]) + "\">" + files[i] + "</a>";
-
-							// Show file size
-							if (curFile.isFile()) {
-								long len = curFile.length();
-								msg += " &nbsp;<font size=2>(";
-								if (len < 1024)
-									msg += len + " bytes";
-								else if (len < 1024 * 1024)
-									msg += len / 1024 + "." + (len % 1024 / 10 % 100) + " KB";
-								else
-									msg += len / (1024 * 1024) + "." + len % (1024 * 1024) / 10 % 100 + " MB";
-
-								msg += ")</font>";
-							}
-							msg += "<br/>";
-							if (dir)
-								msg += "</b>";
+					for (String name : IO.list(f)) {
+						File curFile = new File(f, name);
+						boolean dir = curFile.isDirectory();
+						if (dir) {
+							msg += "<b>";
+							name += "/";
 						}
+
+						msg += "<a href=\"" + encodeUri(uri + name) + "\">" + name + "</a>";
+
+						// Show file size
+						if (curFile.isFile()) {
+							long len = curFile.length();
+							msg += " &nbsp;<font size=2>(";
+							if (len < 1024)
+								msg += len + " bytes";
+							else if (len < 1024 * 1024)
+								msg += len / 1024 + "." + (len % 1024 / 10 % 100) + " KB";
+							else
+								msg += len / (1024 * 1024) + "." + len % (1024 * 1024) / 10 % 100 + " MB";
+
+							msg += ")</font>";
+						}
+						msg += "<br/>";
+						if (dir)
+							msg += "</b>";
 					}
 					msg += "</body></html>";
 					res = new Response(HTTP_OK, MIME_HTML, msg);
@@ -872,7 +869,7 @@ public class NanoHTTPD {
 				if (dot >= 0)
 					mime = theMimeTypes.get(f.getCanonicalPath()
 						.substring(dot + 1)
-						.toLowerCase());
+						.toLowerCase(Locale.ROOT));
 				if (mime == null)
 					mime = MIME_DEFAULT_BINARY;
 
