@@ -1,11 +1,13 @@
 package aQute.bnd.signing;
 
+import static aQute.bnd.osgi.Jar.METAINF_SIGNING_P;
+
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.InputStream;
 import java.util.Map;
 import java.util.concurrent.TimeUnit;
-import java.util.regex.Pattern;
+import java.util.jar.JarFile;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -13,7 +15,10 @@ import org.slf4j.LoggerFactory;
 import aQute.bnd.osgi.Builder;
 import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.Jar;
+import aQute.bnd.osgi.Processor;
 import aQute.bnd.service.Plugin;
+import aQute.bnd.service.Registry;
+import aQute.bnd.service.RegistryPlugin;
 import aQute.bnd.service.SignerPlugin;
 import aQute.bnd.stream.MapStream;
 import aQute.lib.io.IO;
@@ -27,8 +32,8 @@ import aQute.service.reporter.Reporter;
  * @author aqute
  */
 
-@aQute.bnd.annotation.plugin.BndPlugin(name = "signer", parameters = JartoolSigner.Config.class)
-public class JartoolSigner implements Plugin, SignerPlugin {
+@aQute.bnd.annotation.plugin.BndPlugin(name = "Signer", parameters = JartoolSigner.Config.class)
+public class JartoolSigner implements Plugin, SignerPlugin, RegistryPlugin {
 	private final static Logger logger = LoggerFactory.getLogger(JartoolSigner.class);
 
 	@interface Config {
@@ -64,6 +69,8 @@ public class JartoolSigner implements Plugin, SignerPlugin {
 	String	tsacert;
 	String	tsapolicyid;
 
+	private Processor	processor;
+
 	@Override
 	public void setProperties(Map<String, String> map) {
 		if (map.containsKey("keystore"))
@@ -91,7 +98,10 @@ public class JartoolSigner implements Plugin, SignerPlugin {
 	@Override
 	public void setReporter(Reporter processor) {}
 
-	private static Pattern EXTENSIONS_P = Pattern.compile(".*\\.(DSA|RSA|SF|MF)$");
+	@Override
+	public void setRegistry(Registry registry) {
+		processor = registry.getPlugin(Processor.class);
+	}
 
 	@Override
 	public void sign(Builder builder, String alias) throws Exception {
@@ -102,12 +112,19 @@ public class JartoolSigner implements Plugin, SignerPlugin {
 		}
 
 		Jar jar = builder.getJar();
-		File tmp = File.createTempFile("signdjar", ".jar");
+		File tmp = File.createTempFile("signedjar", ".jar");
 		tmp.deleteOnExit();
 
 		jar.write(tmp);
 
 		Command command = new Command();
+		if ((path == null) || path.equals("jarsigner")) {
+			if (processor != null) {
+				path = processor.getJavaExecutable("jarsigner");
+			} else {
+				path = IO.getJavaExecutablePath("jarsigner");
+			}
+		}
 		command.add(path);
 		if (keystore != null) {
 			command.add("-keystore");
@@ -171,7 +188,7 @@ public class JartoolSigner implements Plugin, SignerPlugin {
 		builder.addClose(signed);
 
 		MapStream.of(signed.getDirectory("META-INF"))
-			.filterKey(path -> EXTENSIONS_P.matcher(path)
+			.filterKey(path -> JarFile.MANIFEST_NAME.equals(path) || METAINF_SIGNING_P.matcher(path)
 				.matches())
 			.forEachOrdered(jar::putResource);
 		jar.setDoNotTouchManifest();
