@@ -18,8 +18,8 @@ import org.osgi.framework.Version;
 import org.osgi.namespace.extender.ExtenderNamespace;
 import org.osgi.service.configurator.ConfiguratorConstants;
 
+import aQute.bnd.exceptions.Exceptions;
 import aQute.bnd.service.specifications.BuilderSpecification;
-import aQute.lib.exceptions.Exceptions;
 import aQute.lib.io.IO;
 import aQute.lib.regex.PatternConstants;
 import aQute.lib.strings.Strings;
@@ -119,7 +119,8 @@ public interface BundleSpecBuilder {
 		 * @param value the value
 		 */
 		default BundleSpecBsn attribute(String key, String value) {
-			x().add("Bundle-SymbolicName", x().spec.bundleSymbolicName, bsn(), key, value);
+			BundleBuilder x = x();
+			x.add("Bundle-SymbolicName", x.spec.bundleSymbolicName, bsn(), key, value);
 			return this;
 		}
 
@@ -145,10 +146,11 @@ public interface BundleSpecBuilder {
 	 * Bundle-Version
 	 */
 	default BundleSpecBuilder bundleVersion(String version) {
-		if (x().spec.bundleVersion != null) {
-			throw new IllegalArgumentException("Bundle-Version was already set to: " + x().spec.bundleVersion);
+		BundleBuilder x = x();
+		if (x.spec.bundleVersion != null) {
+			throw new IllegalArgumentException("Bundle-Version was already set to: " + x.spec.bundleVersion);
 		}
-		x().spec.bundleVersion = version;
+		x.spec.bundleVersion = version;
 		new Version(version);
 		return this;
 	}
@@ -157,10 +159,11 @@ public interface BundleSpecBuilder {
 	 * Bundle-Activator
 	 */
 	default BundleSpecBuilder bundleActivator(String bundleActivator) {
-		if (x().spec.bundleActivator != null) {
-			throw new IllegalArgumentException("Bundle-Activator was already set to: " + x().spec.bundleActivator);
+		BundleBuilder x = x();
+		if (x.spec.bundleActivator != null) {
+			throw new IllegalArgumentException("Bundle-Activator was already set to: " + x.spec.bundleActivator);
 		}
-		x().spec.bundleActivator = bundleActivator;
+		x.spec.bundleActivator = bundleActivator;
 		return this;
 	}
 
@@ -252,7 +255,8 @@ public interface BundleSpecBuilder {
 		 * @param value the attribute value
 		 */
 		default BundleSpecFragmentHost attribute(String key, String value) {
-			x().add("Fragment-Host", x().spec.fragmentHost, fragmenthost(), key, value);
+			BundleBuilder x = x();
+			x.add("Fragment-Host", x.spec.fragmentHost, fragmenthost(), key, value);
 			return this;
 		}
 
@@ -279,7 +283,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecBuilderRequireBundle requireBundle(String name) {
 		BundleBuilder x = x();
-		String actualName = x().prepare(name, x.spec.requireBundle);
+		String actualName = x.prepare(name, x.spec.requireBundle);
 
 		return new BundleSpecBuilderRequireBundle() {
 
@@ -340,7 +344,8 @@ public interface BundleSpecBuilder {
 		 * @param value the value of the attribute
 		 */
 		default BundleSpecBuilderRequireBundle attribute(String key, String value) {
-			x().add("Require-Bundle ", x().spec.requireBundle, requirebundle(), key, value);
+			BundleBuilder x = x();
+			x.add("Require-Bundle ", x.spec.requireBundle, requirebundle(), key, value);
 			return this;
 		}
 
@@ -364,7 +369,8 @@ public interface BundleSpecBuilder {
 	/**
 	 * Import-Package
 	 *
-	 * @param name name of the package to be imported, may contain wildcards
+	 * @param packageName name of the package to be imported, may contain
+	 *            wildcards
 	 */
 
 	default BundleSpecImportPackage importPackage(String packageName) {
@@ -441,7 +447,8 @@ public interface BundleSpecBuilder {
 		 * @param value the value of the attribute
 		 */
 		default BundleSpecImportPackage attribute(String key, String value) {
-			x().add("Import-Package", x().spec.importPackage, packageName(), key, value);
+			BundleBuilder x = x();
+			x.add("Import-Package", x.spec.importPackage, packageName(), key, value);
 			return this;
 		}
 
@@ -469,7 +476,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecExportPackage exportPackage(String name) {
 		BundleBuilder x = x();
-		String packageName = x().prepare(name, x.spec.exportPackage);
+		String packageName = x.prepare(name, x.spec.exportPackage);
 
 		return new BundleSpecExportPackage() {
 
@@ -530,9 +537,9 @@ public interface BundleSpecBuilder {
 		/**
 		 * Set included classes
 		 *
-		 * @param classNames the included class names
+		 * @param classes the included class names
 		 */
-		default BundleSpecExportPackage include(@SuppressWarnings("rawtypes") Class... classes) {
+		default BundleSpecExportPackage include(Class<?>... classes) {
 			return this.include(Stream.of(classes)
 				.map(Class::getName)
 				.toArray(String[]::new));
@@ -549,7 +556,8 @@ public interface BundleSpecBuilder {
 		}
 
 		default BundleSpecExportPackage attribute(String key, String value) {
-			x().add("Export-Package", x().spec.exportPackage, packageName(), key, value);
+			BundleBuilder x = x();
+			x.add("Export-Package", x.spec.exportPackage, packageName(), key, value);
 			return this;
 		}
 
@@ -568,7 +576,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecExportContents exportContent(String name) {
 		BundleBuilder x = x();
-		String actualName = x().prepare(name, x.spec.exportContents);
+		String actualName = x.prepare(name, x.spec.exportContents);
 
 		return new BundleSpecExportContents() {
 
@@ -613,7 +621,8 @@ public interface BundleSpecBuilder {
 		}
 
 		default BundleSpecExportContents attribute(String key, String value) {
-			x().add("-exportcontents", x().spec.exportPackage, packageName(), key, value);
+			BundleBuilder x = x();
+			x.add("-exportcontents", x.spec.exportPackage, packageName(), key, value);
 			return this;
 		}
 
@@ -629,7 +638,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecBuilder privatePackage(String name) {
 		BundleBuilder x = x();
-		x().prepare(name, x.spec.privatePackage);
+		x.prepare(name, x.spec.privatePackage);
 		return this;
 	}
 
@@ -639,7 +648,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecProvideCapability provideCapability(String namespace) {
 		BundleBuilder x = x();
-		String actualNamespace = x().prepare(namespace, x.spec.exportPackage);
+		String actualNamespace = x.prepare(namespace, x.spec.exportPackage);
 
 		return new BundleSpecProvideCapability() {
 
@@ -679,7 +688,8 @@ public interface BundleSpecBuilder {
 		}
 
 		default BundleSpecProvideCapability attribute(String key, String value) {
-			x().add("Provide-Capability", x().spec.provideCapability, namespace(), key, value);
+			BundleBuilder x = x();
+			x.add("Provide-Capability", x.spec.provideCapability, namespace(), key, value);
 			return this;
 		}
 
@@ -695,7 +705,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecRequireCapability requireCapability(String namespace) {
 		BundleBuilder x = x();
-		String actualNamespace = x().prepare(namespace, x.spec.exportPackage);
+		String actualNamespace = x.prepare(namespace, x.spec.exportPackage);
 
 		return new BundleSpecRequireCapability() {
 
@@ -735,7 +745,8 @@ public interface BundleSpecBuilder {
 		}
 
 		default BundleSpecRequireCapability attribute(String key, String value) {
-			x().add("Require-Capability", x().spec.requireCapability, namespace(), key, value);
+			BundleBuilder x = x();
+			x.add("Require-Capability", x.spec.requireCapability, namespace(), key, value);
 			return this;
 		}
 
@@ -762,7 +773,7 @@ public interface BundleSpecBuilder {
 
 	default BundleSpecIncludeResource includeResource(String spec) {
 		BundleBuilder x = x();
-		String actualSpec = x().prepare(spec, x().spec.includeresource);
+		String actualSpec = x.prepare(spec, x.spec.includeresource);
 
 		return new BundleSpecIncludeResource() {
 
@@ -795,11 +806,11 @@ public interface BundleSpecBuilder {
 	}
 
 	default BundleSpecIncludeResource include(String target, String source) {
-		return includeResource(null, source, false, false);
+		return includeResource(target, source, false, false);
 	}
 
 	default BundleSpecIncludeResource includeIfPresent(String target, String source) {
-		return includeResource(null, source, false, true);
+		return includeResource(target, source, false, true);
 	}
 
 	default BundleSpecIncludeResource preprocessIfPresent(String target, String source) {
@@ -807,7 +818,7 @@ public interface BundleSpecBuilder {
 	}
 
 	default BundleSpecIncludeResource preprocess(String target, String source) {
-		return includeResource(null, source, true, false);
+		return includeResource(target, source, true, false);
 	}
 
 	interface BundleSpecIncludeResource extends BundleSpecBuilder {
@@ -819,7 +830,8 @@ public interface BundleSpecBuilder {
 		}
 
 		default BundleSpecIncludeResource attribute(String key, String value) {
-			x().add("-includeresource", x().spec.includeresource, spec(), key, value);
+			BundleBuilder x = x();
+			x.add("-includeresource", x.spec.includeresource, spec(), key, value);
 			return this;
 		}
 
@@ -830,22 +842,26 @@ public interface BundleSpecBuilder {
 	}
 
 	default BundleSpecBuilder failOk() {
-		x().spec.failOk = true;
+		BundleBuilder x = x();
+		x.spec.failOk = true;
 		return this;
 	}
 
 	default BundleSpecBuilder sources() {
-		x().spec.sources = true;
+		BundleBuilder x = x();
+		x.spec.sources = true;
 		return this;
 	}
 
 	default BundleSpecBuilder instruction(String instruction, String value) {
-		x().spec.other.put(instruction, value);
+		BundleBuilder x = x();
+		String put = x.spec.other.put(instruction, value);
 		return this;
 	}
 
 	default BundleSpecBuilder header(String header, String value) {
-		x().spec.other.put(header, value);
+		BundleBuilder x = x();
+		x.spec.other.put(header, value);
 		return this;
 	}
 
@@ -857,7 +873,8 @@ public interface BundleSpecBuilder {
 	}
 
 	default BundleSpecBuilder classpath(String path) {
-		x().spec.classpath.add(path);
+		BundleBuilder x = x();
+		x.spec.classpath.add(path);
 		return this;
 	}
 
@@ -906,7 +923,7 @@ public interface BundleSpecBuilder {
 			if ("file".equals(url.getProtocol())) {
 				f = new File(url.toURI());
 			} else {
-				f = File.createTempFile("xx", "conf");
+				f = File.createTempFile("bundlespecbuilder", "conf");
 				IO.store(url.openStream(), f);
 			}
 			x().addClose(() -> IO.delete(f));
@@ -969,8 +986,9 @@ public interface BundleSpecBuilder {
 	}
 
 	default Bundle install() throws Exception {
-		BuilderSpecification spec = x().spec;
-		byte[] build = LaunchpadBuilder.workspace.build(LaunchpadBuilder.projectDir.getAbsolutePath(), spec);
+		BundleBuilder x = x();
+		BuilderSpecification spec = x.spec;
+		byte[] build = x.ws.workspace.build(LaunchpadBuilder.projectDir.getAbsolutePath(), spec);
 		String location;
 		if (spec.location == null) {
 			String name = spec.bundleSymbolicName.toString();
@@ -982,14 +1000,15 @@ public interface BundleSpecBuilder {
 		}
 
 		ByteArrayInputStream bin = new ByteArrayInputStream(build);
-		return x().ws.getBundleContext()
+		return x.ws.getBundleContext()
 			.installBundle(location, bin);
 	}
 
 	default Bundle start() {
 		try {
 			Bundle b = install();
-			x().ws.start(b);
+			BundleBuilder x = x();
+			x.ws.start(b);
 			return b;
 		} catch (Exception e) {
 			throw Exceptions.duck(e);
@@ -997,7 +1016,8 @@ public interface BundleSpecBuilder {
 	}
 
 	default BundleSpecBuilder resourceOnly() {
-		x().spec.resourceOnly = true;
+		BundleBuilder x = x();
+		x.spec.resourceOnly = true;
 		return this;
 	}
 
@@ -1006,7 +1026,8 @@ public interface BundleSpecBuilder {
 	 */
 	@Deprecated
 	default BundleSpecBuilder inherit() {
-		x().spec.parent.add(BuilderSpecification.PROJECT);
+		BundleBuilder x = x();
+		x.spec.parent.add(BuilderSpecification.PROJECT);
 		return this;
 	}
 
@@ -1033,7 +1054,8 @@ public interface BundleSpecBuilder {
 	 */
 	default BundleSpecBuilder parent(String path) {
 
-		List<String> paths = x().spec.parent;
+		BundleBuilder x = x();
+		List<String> paths = x.spec.parent;
 		String last = paths.isEmpty() ? null : paths.get(paths.size() - 1);
 
 		boolean isTerminatedWithProjectOrWorkspace = PROJECT.equals(last) || WORKSPACE.equals(last);
@@ -1045,13 +1067,13 @@ public interface BundleSpecBuilder {
 		boolean isFileParent = !WORKSPACE.equals(path) && !PROJECT.equals(path);
 
 		if (isFileParent) {
-			File f = IO.getFile(x().ws.projectDir, path);
+			File f = IO.getFile(x.ws.projectDir, path);
 			if (!f.isFile())
 				throw new IllegalArgumentException("No such parent file " + f.getAbsolutePath());
 
 			path = f.getAbsolutePath();
 		}
-		x().spec.parent.add(path);
+		x.spec.parent.add(path);
 		return this;
 	}
 
@@ -1067,7 +1089,8 @@ public interface BundleSpecBuilder {
 	 * @return The builder object for chained invocations.
 	 */
 	default BundleSpecBuilder location(String location) {
-		x().spec.location = location;
+		BundleBuilder x = x();
+		x.spec.location = location;
 		return this;
 	}
 }
