@@ -17,7 +17,6 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.Dictionary;
 import java.util.EnumSet;
-import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.LinkedList;
 import java.util.List;
@@ -45,6 +44,7 @@ import aQute.bnd.http.HttpClient;
 import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.repository.BaseRepository;
 import aQute.bnd.osgi.resource.CapReqBuilder;
+import aQute.bnd.osgi.resource.ResourceUtils;
 import aQute.bnd.service.IndexProvider;
 import aQute.bnd.service.Plugin;
 import aQute.bnd.service.Refreshable;
@@ -423,13 +423,11 @@ public abstract class AbstractIndexedRepo extends BaseRepository
 			throw new RuntimeException(e);
 		}
 
-		Map<Requirement, Collection<Capability>> result = new HashMap<>();
-		for (Requirement requirement : requirements) {
-			List<Capability> matches = new LinkedList<>();
-			result.put(requirement, matches);
-
+		Map<Requirement, Collection<Capability>> result = ResourceUtils.findProviders(requirements, requirement -> {
+			List<Capability> matches = new ArrayList<>();
 			capabilityIndex.appendMatchingCapabilities(requirement, matches);
-		}
+			return matches;
+		});
 		return result;
 	}
 
@@ -526,7 +524,7 @@ public abstract class AbstractIndexedRepo extends BaseRepository
 
 	ResourceHandle findExactMatch(String identity, String version) throws Exception {
 		VersionRange range = new VersionRange(version);
-		if (range.isRange())
+		if (!range.isSingleVersion())
 			return null;
 		Resource resource = identityMap.getExact(identity, range.getLow());
 		if (resource == null)
@@ -557,8 +555,8 @@ public abstract class AbstractIndexedRepo extends BaseRepository
 		if (!SHA_256.equalsIgnoreCase(algo))
 			return null;
 
-		String contentFilter = String.format("(%s=%s)", ContentNamespace.CONTENT_NAMESPACE, hashStr);
-		Requirement contentReq = new CapReqBuilder(ContentNamespace.CONTENT_NAMESPACE).filter(contentFilter)
+		Requirement contentReq = CapReqBuilder
+			.createSimpleRequirement(ContentNamespace.CONTENT_NAMESPACE, hashStr, null)
 			.buildSyntheticRequirement();
 
 		List<Capability> caps = new LinkedList<>();
