@@ -4,7 +4,6 @@ import static aQute.bnd.deployer.repository.RepoConstants.DEFAULT_CACHE_DIR;
 
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileFilter;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.MalformedURLException;
@@ -18,18 +17,20 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.LinkedList;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Stream;
 
 import org.osgi.service.coordinator.Coordination;
 import org.osgi.service.coordinator.Coordinator;
 import org.osgi.service.coordinator.Participant;
 import org.osgi.service.log.LogService;
 
+import aQute.bnd.annotation.plugin.BndPlugin;
 import aQute.bnd.deployer.repository.api.IRepositoryContentProvider;
 import aQute.bnd.osgi.Jar;
 import aQute.bnd.osgi.Verifier;
@@ -46,35 +47,49 @@ import aQute.lib.strings.Strings;
 import aQute.libg.cryptography.SHA1;
 import aQute.libg.cryptography.SHA256;
 
+@BndPlugin(name = "LocalIndexedRepo", parameters = LocalIndexedRepo.Config.class)
 public class LocalIndexedRepo extends AbstractIndexedRepo implements Refreshable, Participant, Actionable {
 
-	private final String			UPWARDS_ARROW			= " \u2191";
-	private final String			DOWNWARDS_ARROW			= " \u2193";
-	private static final Pattern	REPO_FILE				= Pattern
+	private final String			UPWARDS_ARROW	= " \u2191";
+	private final String			DOWNWARDS_ARROW	= " \u2193";
+	private static final Pattern	REPO_FILE		= Pattern
 		.compile("([-.\\w]+)(-|_)([.\\d]+)(-[-\\w]+)?\\.(jar|lib)");
-	private static final String		CACHE_PATH				= ".cache";
-	public static final String		PROP_LOCAL_DIR			= "local";
-	public static final String		PROP_READONLY			= "readonly";
-	public static final String		PROP_PRETTY				= "pretty";
-	public static final String		PROP_OVERWRITE			= "overwrite";
-	public static final String		PROP_ONLYDIRS			= "onlydirs";
+	private static final String		CACHE_PATH		= ".cache";
+	public static final String		PROP_LOCAL_DIR	= "local";
+	public static final String		PROP_READONLY	= "readonly";
+	public static final String		PROP_PRETTY		= "pretty";
+	public static final String		PROP_OVERWRITE	= "overwrite";
+	public static final String		PROP_ONLYDIRS	= "onlydirs";
+
+	// not actually used (yet). Just to get some parameters
+	interface Config {
+		String local();
+
+		boolean readonly();
+
+		boolean pretty();
+
+		boolean overwrite();
+
+		boolean onlydirs();
+	}
 
 	@SuppressWarnings("deprecation")
-	private boolean					readOnly;
-	private boolean					pretty					= false;
-	private boolean					overwrite				= true;
-	private File					storageDir;
-	private String					onlydirs				= null;
+	private boolean				readOnly;
+	private boolean				pretty					= false;
+	private boolean				overwrite				= true;
+	private File				storageDir;
+	private String				onlydirs				= null;
 
 	// @GuardedBy("newFilesInCoordination")
-	private final List<URI>			newFilesInCoordination	= new LinkedList<>();
-	private static final String		EMPTY_LOCATION			= "";
+	private final List<URI>		newFilesInCoordination	= new ArrayList<>();
+	private static final String	EMPTY_LOCATION			= "";
 
-	public static final String		PROP_LOCATIONS			= "locations";
-	public static final String		PROP_CACHE				= "cache";
+	public static final String	PROP_LOCATIONS			= "locations";
+	public static final String	PROP_CACHE				= "cache";
 
-	private String					locations;
-	protected File					cacheDir				= new File(
+	private String				locations;
+	protected File				cacheDir				= new File(
 		System.getProperty("user.home") + File.separator + DEFAULT_CACHE_DIR);
 
 	@SuppressWarnings("deprecation")
@@ -238,7 +253,7 @@ public class LocalIndexedRepo extends AbstractIndexedRepo implements Refreshable
 			MessageDigest md = MessageDigest.getInstance(SHA256.ALGORITHM);
 			md.update(data);
 			IO.store(Hex.toHexString(md.digest())
-				.toLowerCase(), shaFile);
+				.toLowerCase(Locale.ROOT), shaFile);
 		}
 	}
 
@@ -247,7 +262,7 @@ public class LocalIndexedRepo extends AbstractIndexedRepo implements Refreshable
 		if (!storageDir.isDirectory())
 			return;
 
-		LinkedList<File> files = new LinkedList<>();
+		List<File> files = new ArrayList<>();
 		String[] onlydirsFiles = null;
 		if (onlydirs != null) {
 			String[] onlydirs2 = onlydirs.split(",");
@@ -263,32 +278,35 @@ public class LocalIndexedRepo extends AbstractIndexedRepo implements Refreshable
 	}
 
 	private void listRecurse(final Pattern pattern, final String[] onlydirsFiles, File root, File dir,
-		LinkedList<File> files) {
-		final LinkedList<File> dirs = new LinkedList<>();
-		File[] moreFiles = dir.listFiles((FileFilter) f -> {
-			if (f.isDirectory()) {
-				boolean addit = true;
-				if (onlydirsFiles != null) {
-					String fabs = f.getAbsolutePath();
-					addit = false;
-					for (String dirtest : onlydirsFiles) {
-						if (dirtest.startsWith(fabs) || fabs.startsWith(dirtest)) {
-							addit = true;
-							break;
+		List<File> files) {
+		final List<File> dirs = new ArrayList<>();
+		try (Stream<String> names = IO.listStream(dir)) {
+			names.map(name -> new File(dir, name))
+				.filter(f -> {
+					if (f.isDirectory()) {
+						boolean addit = true;
+						if (onlydirsFiles != null) {
+							String fabs = f.getAbsolutePath();
+							addit = false;
+							for (String dirtest : onlydirsFiles) {
+								if (dirtest.startsWith(fabs) || fabs.startsWith(dirtest)) {
+									addit = true;
+									break;
+								}
+							}
 						}
+						if (addit) {
+							dirs.add(f);
+						}
+					} else if (f.isFile()) {
+						Matcher matcher = pattern.matcher(f.getName());
+						return matcher.matches();
 					}
-				}
-				if (addit) {
-					dirs.add(f);
-				}
-			} else if (f.isFile()) {
-				Matcher matcher = pattern.matcher(f.getName());
-				return matcher.matches();
-			}
-			return false;
-		});
-		// Add the files that we found.
-		files.addAll(Arrays.asList(moreFiles));
+					return false;
+				})
+				// Add the files that we found.
+				.forEachOrdered(files::add);
+		}
 
 		// keep recursing
 		for (File d : dirs) {
@@ -494,7 +512,7 @@ public class LocalIndexedRepo extends AbstractIndexedRepo implements Refreshable
 	@Override
 	public Map<String, Runnable> actions(Object... target) throws Exception {
 		Map<String, Runnable> map = new HashMap<>();
-		map.put("Refresh", () -> regenerateAllIndexes());
+		map.put("Refresh", this::regenerateAllIndexes);
 		if (target.length == 3) {
 			String bsn = (String) target[1];
 			String version = (String) target[2];
