@@ -3,17 +3,21 @@ package aQute.bnd.deployer.repository.providers;
 import static aQute.bnd.deployer.repository.api.Decision.accept;
 import static aQute.bnd.deployer.repository.api.Decision.reject;
 import static aQute.bnd.deployer.repository.api.Decision.undecided;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.ByteArrayInputStream;
 
 import javax.xml.stream.XMLStreamException;
 
+import org.junit.jupiter.api.Test;
+
 import aQute.bnd.deployer.repository.api.CheckResult;
-import junit.framework.TestCase;
 
-public class TestR5Recognition extends TestCase {
+public class TestR5Recognition {
 
-	public static void testRejectNamespace() throws Exception {
+	@Test
+	public void testRejectNamespace() throws Exception {
 		String testdata = "<?xml version='1.0' encoding='utf-8'?>"
 			+ "<repository increment='0' name='index1' xmlns='http://www2.osgi.org/www/obr2html.xsl'>" + "<resource>";
 		ByteArrayInputStream stream = new ByteArrayInputStream(testdata.getBytes());
@@ -21,21 +25,24 @@ public class TestR5Recognition extends TestCase {
 			.getDecision());
 	}
 
-	public static void testAcceptNamespace() throws Exception {
+	@Test
+	public void testAcceptNamespace() throws Exception {
 		String testdata = "<?xml version='1.0'?>" + "<repository xmlns='http://www.osgi.org/xmlns/repository/v1.0.0'>";
 		ByteArrayInputStream stream = new ByteArrayInputStream(testdata.getBytes());
 		assertEquals(accept, new R5RepoContentProvider().checkStream("xxx", stream)
 			.getDecision());
 	}
 
-	public static void testRejectRootElementName() throws Exception {
+	@Test
+	public void testRejectRootElementName() throws Exception {
 		String testdata = "<?xml version='1.0' encoding='utf-8'?>" + "<repo name='index1'/>";
 		ByteArrayInputStream stream = new ByteArrayInputStream(testdata.getBytes());
 		assertEquals(reject, new R5RepoContentProvider().checkStream("xxx", stream)
 			.getDecision());
 	}
 
-	public static void testUndecidable() throws Exception {
+	@Test
+	public void testUndecidable() throws Exception {
 		String testdata;
 		ByteArrayInputStream stream;
 		CheckResult result;
@@ -56,7 +63,8 @@ public class TestR5Recognition extends TestCase {
 		assertEquals(undecided, result.getDecision());
 	}
 
-	public static void testUnparseable() throws Exception {
+	@Test
+	public void testUnparseable() throws Exception {
 		String testdata = "<?xml version='1.0' encoding='utf-8'?>" + "<repository name='index1'>";
 		ByteArrayInputStream stream = new ByteArrayInputStream(testdata.getBytes());
 		CheckResult result = new R5RepoContentProvider().checkStream("xxx", stream);
@@ -64,7 +72,8 @@ public class TestR5Recognition extends TestCase {
 		assertTrue(result.getException() != null && result.getException() instanceof XMLStreamException);
 	}
 
-	public static void testAcceptOnCapabilityChildElementNames() throws Exception {
+	@Test
+	public void testAcceptOnCapabilityChildElementNames() throws Exception {
 		String testdata;
 		ByteArrayInputStream stream;
 		CheckResult result;
@@ -88,7 +97,8 @@ public class TestR5Recognition extends TestCase {
 		assertEquals(accept, result.getDecision());
 	}
 
-	public static void testAcceptExtensionElementOtherNamespace() throws Exception {
+	@Test
+	public void testAcceptExtensionElementOtherNamespace() throws Exception {
 		String testdata;
 		ByteArrayInputStream stream;
 		CheckResult result;
