@@ -1,6 +1,11 @@
 package biz.aQute.resolve;
 
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 import static test.lib.Utils.createRepo;
 
 import java.io.File;
@@ -15,6 +20,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 import org.osgi.framework.Version;
 import org.osgi.framework.namespace.IdentityNamespace;
@@ -42,21 +48,24 @@ import aQute.bnd.osgi.resource.ResourceBuilder;
 import aQute.bnd.osgi.resource.ResourceUtils;
 import aQute.bnd.osgi.resource.ResourceUtils.IdentityCapability;
 import aQute.bnd.repository.osgi.OSGiRepository;
+import aQute.bnd.test.jupiter.InjectTemporaryDirectory;
 import aQute.lib.dot.DOT;
 import aQute.lib.io.IO;
 import aQute.libg.reporter.ReporterAdapter;
-import junit.framework.TestCase;
 import test.lib.MockRegistry;
 
 @SuppressWarnings({
 	"restriction", "deprecation"
 })
-public class ResolveTest extends TestCase {
+public class ResolveTest {
 
 	private static final LogService log = new LogReporter(new ReporterAdapter(System.out));
 
+	@InjectTemporaryDirectory
+	File							tmp;
+
 	private String getTestName() {
-		return getClass().getName() + "/" + getName();
+		return tmp.getName();
 	}
 
 	/**
@@ -69,6 +78,7 @@ public class ResolveTest extends TestCase {
 	 * <p>
 	 * Tried this but seems to work in all combinations as expected
 	 */
+	@Test
 	public void testIncludeBndrun() throws Exception {
 		assertInclude("intop.bndrun", "top");
 		assertInclude("ininclude.bndrun", "include");
@@ -102,6 +112,7 @@ public class ResolveTest extends TestCase {
 	 * Missing default version
 	 */
 
+	@Test
 	public void testDefaultVersionsForJava() throws Exception {
 		Run run = Run.createRun(null, IO.getFile("testdata/defltversions/run.bndrun"));
 		try (Workspace w = run.getWorkspace(); ProjectResolver pr = new ProjectResolver(run);) {
@@ -118,9 +129,10 @@ public class ResolveTest extends TestCase {
 	 * not run
 	 */
 
+	@Test
 	public void testenRouteGuard() throws Exception {
 		MockRegistry registry = new MockRegistry();
-		Repository repo = createRepo(IO.getFile("testdata/enroute/index.xml"), getTestName());
+		Repository repo = createRepo(IO.getFile("testdata/enroute/index.xml"), getTestName(), tmp);
 		registry.addPlugin(repo);
 
 		List<Requirement> reqs = CapReqBuilder.getRequirementsFrom(
@@ -163,6 +175,7 @@ public class ResolveTest extends TestCase {
 	 * @throws Exception
 	 */
 
+	@Test
 	public void testResolveWithAugments() throws Exception {
 		// Add requirement
 		assertAugmentResolve(
@@ -204,7 +217,7 @@ public class ResolveTest extends TestCase {
 	private void assertAugmentResolve(String augment, String require, String effective) throws Exception {
 
 		MockRegistry registry = new MockRegistry();
-		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName()));
+		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName(), tmp));
 
 		Processor model = new Processor();
 		model.setRunfw("org.apache.felix.framework");
@@ -230,6 +243,7 @@ public class ResolveTest extends TestCase {
 	 * @throws URISyntaxException
 	 * @throws MalformedURLException
 	 */
+	@Test
 	public void testMinimalSetup() throws Exception {
 		try (OSGiRepository repo = new OSGiRepository(); HttpClient httpClient = new HttpClient()) {
 			Map<String, String> map = new HashMap<>();
@@ -237,8 +251,7 @@ public class ResolveTest extends TestCase {
 				.toURI()
 				.toString());
 			map.put("name", getTestName());
-			map.put("cache",
-				new File("generated/tmp/test/cache/" + getTestName()).getAbsolutePath());
+			map.put("cache", new File(tmp, "cache").getAbsolutePath());
 			repo.setProperties(map);
 			Processor model = new Processor();
 			model.addBasicPlugin(httpClient);
@@ -268,10 +281,11 @@ public class ResolveTest extends TestCase {
 	 *
 	 * @throws ResolutionException
 	 */
+	@Test
 	public void testResolveWithDistro() throws Exception {
 
 		MockRegistry registry = new MockRegistry();
-		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName()));
+		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName(), tmp));
 
 		BndEditModel model = new BndEditModel();
 		model.setDistro(Arrays.asList("testdata/distro.jar;version=file"));
@@ -298,10 +312,11 @@ public class ResolveTest extends TestCase {
 	 *
 	 * @throws ResolutionException
 	 */
+	@Test
 	public void testResolveWithLargeDistro() throws Exception {
 
 		MockRegistry registry = new MockRegistry();
-		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName()));
+		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName(), tmp));
 
 		BndEditModel model = new BndEditModel();
 		model.setDistro(Arrays.asList("testdata/release.dxp.distro-7.2.10.jar;version=file"));
@@ -327,6 +342,7 @@ public class ResolveTest extends TestCase {
 	 * Test if we can resolve fragment with a distro without whitelisting the
 	 * 'osgi.wiring.host' capabilities on the system resource. Should FAIL
 	 */
+	@Test
 	public void testResolveFragmentWithDistro_FAIL() throws Exception {
 		File f = IO.getFile("testdata/repo9/fragment.bndrun");
 		File distro = IO.getFile("testdata/release.dxp.distro-7.2.10.jar");
@@ -350,6 +366,7 @@ public class ResolveTest extends TestCase {
 	 * Test if we can resolve fragment with a distro when whitelisting the
 	 * 'osgi.wiring.host' capabilities on the system resource. Should NOT fail
 	 */
+	@Test
 	public void testResolveFragmentWithDistro() throws Exception {
 		File f = IO.getFile("testdata/repo9/fragment.bndrun");
 		File distro = IO.getFile("testdata/release.dxp.distro-7.2.10.jar");
@@ -379,7 +396,7 @@ public class ResolveTest extends TestCase {
 		}
 	}
 
-	// public void testResolveWithLargeDistroRepeated() throws Exception {
+	// @Test public void testResolveWithLargeDistroRepeated() throws Exception {
 	// for (int i = 0; i < 1000; i++) {
 	// System.out.println("iteration " + i);
 	// testResolveWithLargeDistro();
@@ -392,10 +409,11 @@ public class ResolveTest extends TestCase {
 	 * this is done in the same way. The {@link #testResolveWithDistro()} has a
 	 * negative check while this one checks positive.
 	 */
+	@Test
 	public void testSimpleResolve() throws Exception {
 
 		MockRegistry registry = new MockRegistry();
-		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName()));
+		registry.addPlugin(createRepo(IO.getFile("testdata/repo3.index.xml"), getTestName(), tmp));
 
 		BndEditModel model = new BndEditModel();
 
@@ -492,7 +510,7 @@ public class ResolveTest extends TestCase {
 	private static Resource getResource(Set<Resource> resources, String bsn, String versionString) {
 		for (Resource resource : resources) {
 			List<Capability> identities = resource.getCapabilities(IdentityNamespace.IDENTITY_NAMESPACE);
-			if (identities != null && identities.size() == 1) {
+			if (identities.size() == 1) {
 				Capability idCap = identities.get(0);
 				Object id = idCap.getAttributes()
 					.get(IdentityNamespace.IDENTITY_NAMESPACE);
@@ -523,11 +541,12 @@ public class ResolveTest extends TestCase {
 	 *
 	 * @throws ResolutionException
 	 */
+	@Test
 	public void testMultipleOptionsNotDuplicated() throws Exception {
 
 		// Resolve against repo 5
 		MockRegistry registry = new MockRegistry();
-		registry.addPlugin(createRepo(IO.getFile("testdata/repo5/index.xml"), getTestName()));
+		registry.addPlugin(createRepo(IO.getFile("testdata/repo5/index.xml"), getTestName(), tmp));
 
 		// Set up a simple Java 7 Felix requirement as per Issue #971
 		BndEditModel runModel = new BndEditModel();
@@ -566,7 +585,7 @@ public class ResolveTest extends TestCase {
 			// this resolve
 			String symbolicName = (String) cap.getAttributes()
 				.get(IdentityNamespace.IDENTITY_NAMESPACE);
-			assertNull("Multiple results for " + symbolicName, mandatoryResourcesBySymbolicName.put(symbolicName, r));
+			assertNull(mandatoryResourcesBySymbolicName.put(symbolicName, r), "Multiple results for " + symbolicName);
 		}
 		assertEquals(4, resolvedResources.size());
 	}
@@ -576,6 +595,7 @@ public class ResolveTest extends TestCase {
 	 *
 	 * @throws Exception
 	 */
+	@Test
 	public void testLatestBundleServiceNamespace() throws Exception {
 		try (OSGiRepository repo = new OSGiRepository(); HttpClient httpClient = new HttpClient()) {
 			Map<String, String> map = new HashMap<>();
@@ -583,8 +603,7 @@ public class ResolveTest extends TestCase {
 				.toURI()
 				.toString());
 			map.put("name", getTestName());
-			map.put("cache",
-				new File("generated/tmp/test/cache/" + getTestName()).getAbsolutePath());
+			map.put("cache", new File(tmp, "cache").getAbsolutePath());
 			repo.setProperties(map);
 			Processor model = new Processor();
 			model.addBasicPlugin(httpClient);
@@ -610,10 +629,10 @@ public class ResolveTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testFrameworkFragmentResolve() throws Exception {
 		String wspath = "framework-fragment/";
-		String genWsPath = "generated/tmp/test/" + getTestName() + "/" + wspath;
-		File wsRoot = IO.getFile(genWsPath);
+		File wsRoot = IO.getFile(tmp, wspath);
 		IO.delete(wsRoot);
 		IO.copy(IO.getFile("testdata/" + wspath), wsRoot);
 
@@ -624,12 +643,13 @@ public class ResolveTest extends TestCase {
 			System.out.println(runbundles);
 			assertThat(bndrun.check()).isTrue();
 			Parameters p = new Parameters(runbundles);
-			assertThat(p.keySet()).hasSize(5)
-				.contains("org.apache.felix.scr", "test.log", "org.apache.felix.log.extension",
+			assertThat(p.keySet()).contains("org.apache.felix.scr", "test.log",
+				"org.apache.felix.log.extension",
 					"org.apache.felix.gogo.command", "org.apache.felix.gogo.runtime");
 		}
 	}
 
+	@Test
 	public void testResolveWithDependencyOrdering() throws Exception {
 		File f = IO.getFile("testdata/enroute/resolver.bndrun");
 		try (Bndrun bndrun = Bndrun.createBndrun(null, f)) {
@@ -640,6 +660,7 @@ public class ResolveTest extends TestCase {
 		}
 	}
 
+	@Test
 	public void testDot() throws Exception {
 		File f = IO.getFile("testdata/enroute/resolver.bndrun");
 		try (Bndrun bndrun = Bndrun.createBndrun(null, f)) {
