@@ -2,10 +2,11 @@ package biz.aQute.resolve;
 
 import java.util.List;
 
-import biz.aQute.resolve.Utils.ResolveTrace;
-import junit.framework.TestCase;
+import org.junit.jupiter.api.Test;
 
-public class UtilsTest extends TestCase {
+import biz.aQute.resolve.Utils.ResolveTrace;
+
+public class UtilsTest {
 	public static String msg1 = "org.osgi.service.resolver.ResolutionException: Unable to resolve"
 		+ " <<INITIAL>> version=null: missing requirement Require[osgi.identity]{}"
 		+ "{filter=(&(osgi.identity=org.apache.felix.gogo.shell)(&(version>=0.0.0)"
@@ -13,6 +14,7 @@ public class UtilsTest extends TestCase {
 		+ "version=0.10.0: missing requirement Require[osgi.wiring.package]{}"
 		+ "{filter=(osgi.wiring.package=org.osgi.framework)}]";
 
+	@Test
 	public void testMessage() {
 		List<ResolveTrace> trace = Utils.parseException(msg1);
 		// assertEquals(2, trace.size());
