@@ -2,6 +2,7 @@ package aQute.tester.bundle.engine.test;
 
 import static aQute.tester.bundle.engine.BundleEngine.CHECK_UNRESOLVED;
 import static org.assertj.core.api.Assertions.allOf;
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.junit.platform.commons.util.FunctionUtils.where;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectClass;
@@ -22,9 +23,12 @@ import static org.junit.platform.testkit.engine.TestExecutionResultConditions.me
 
 import java.io.PrintWriter;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.function.Supplier;
+import java.util.stream.Collectors;
 
+import org.assertj.core.api.AutoCloseableSoftAssertions;
 import org.assertj.core.api.Condition;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
@@ -40,6 +44,7 @@ import org.junit.platform.engine.TestDescriptor;
 import org.junit.platform.engine.TestEngine;
 import org.junit.platform.engine.TestExecutionResult;
 import org.junit.platform.engine.UniqueId;
+import org.junit.platform.engine.discovery.ClassSelector;
 import org.junit.platform.engine.support.descriptor.EngineDescriptor;
 import org.junit.platform.testkit.engine.EngineTestKit;
 import org.junit.platform.testkit.engine.EngineTestKit.Builder;
@@ -47,11 +52,11 @@ import org.junit.platform.testkit.engine.Event;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleException;
 
+import aQute.bnd.exceptions.Exceptions;
 import aQute.bnd.osgi.Constants;
 import aQute.launchpad.BundleSpecBuilder;
 import aQute.launchpad.Launchpad;
 import aQute.launchpad.LaunchpadBuilder;
-import aQute.lib.exceptions.Exceptions;
 import aQute.lib.io.IO;
 import aQute.tester.bundle.engine.BundleDescriptor;
 import aQute.tester.bundle.engine.BundleEngine;
@@ -68,9 +73,11 @@ import aQute.tester.testclasses.bundle.engine.JUnit3And4Test;
 import aQute.tester.testclasses.bundle.engine.JUnit3And5Test;
 import aQute.tester.testclasses.bundle.engine.JUnit3AndVenusTest;
 import aQute.tester.testclasses.bundle.engine.JUnit4Test;
+import aQute.tester.testclasses.bundle.engine.JUnit5ParameterizedSubclassTest;
 import aQute.tester.testclasses.bundle.engine.JUnit5ParameterizedTest;
 import aQute.tester.testclasses.bundle.engine.JUnit5Test;
 import aQute.tester.testclasses.bundle.engine.TestClass;
+import aQute.tester.testclasses.bundle.engine.TestWithField;
 
 @SuppressWarnings("restriction")
 public class BundleEngineTest {
@@ -119,6 +126,14 @@ public class BundleEngineTest {
 		}
 	}
 
+	protected void startLaunchpadNoEngines() {
+		builder = new LaunchpadBuilder();
+		builder.bndrun("bundleenginetest-noengines.bndrun")
+			.excludeExport("aQute.tester.bundle.engine")
+			.excludeExport("aQute.tester.bundle.engine.discovery");
+		startLaunchpad();
+	}
+
 	protected void startLaunchpad() {
 		if (DEBUG) {
 			builder.debug();
@@ -164,7 +179,7 @@ public class BundleEngineTest {
 	public void outsideOfFramework_hasInitializationError() throws Exception {
 		EngineTestKit.engine(new BundleEngine())
 			.execute()
-			.tests()
+			.testEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(test("noFramework"), finishedWithFailure(instanceOf(JUnitException.class),
@@ -174,16 +189,12 @@ public class BundleEngineTest {
 	@Test
 	@Tag(CUSTOM_LAUNCH)
 	public void withNoEngines_reportsMissingEngines_andSkipsMainTests() throws Exception {
-		builder = new LaunchpadBuilder();
-		builder = builder.bndrun("bundleenginetest-noengines.bndrun")
-			.excludeExport("aQute.tester.bundle.engine")
-			.excludeExport("aQute.tester.bundle.engine.discovery");
-		startLaunchpad();
+		startLaunchpadNoEngines();
 
 		Bundle testBundle = testBundler.startTestBundle(JUnit4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -206,7 +217,7 @@ public class BundleEngineTest {
 		startTestBundle(JUnit4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(test("misconfiguredEngines"), unresolvedBundle(engineBundle),
@@ -225,7 +236,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("misconfiguredEngines"), bundle(engineBundle)))
@@ -270,11 +281,7 @@ public class BundleEngineTest {
 	@Test
 	@Tag(CUSTOM_LAUNCH)
 	public void withEngineWithServiceSpecCommentsAndWhitespace_loadsEngine() throws Exception {
-		builder = new LaunchpadBuilder();
-		builder.bndrun("bundleenginetest-noengines.bndrun")
-			.excludeExport("aQute.tester.bundle.engine")
-			.excludeExport("aQute.tester.bundle.engine.discovery");
-		startLaunchpad();
+		startLaunchpadNoEngines();
 
 		Bundle engineBundle = testBundler.bundleWithEE()
 			.includeResource("META-INF/services/" + TestEngine.class.getName())
@@ -285,7 +292,7 @@ public class BundleEngineTest {
 		Bundle testBundle = startTestBundle(JUnit4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("misconfiguredEngines"), bundle(engineBundle), finishedSuccessfully()))
@@ -387,7 +394,7 @@ public class BundleEngineTest {
 		startTestBundle(JUnit4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(lastSegment("engine", BundleEngine.ENGINE_ID), started()))
@@ -408,7 +415,7 @@ public class BundleEngineTest {
 		Bundle testBundle = startTestBundle(JUnit4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unresolvedBundles"), finishedSuccessfully()))
@@ -434,7 +441,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedBundles")))
@@ -451,7 +458,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -468,7 +475,7 @@ public class BundleEngineTest {
 		Bundle resolvedTestBundle = startTestBundle(TestClass.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedBundles")))
@@ -483,7 +490,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectMethod(JUnit5Test.class.getName(), "thisIsBTest"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -501,7 +508,7 @@ public class BundleEngineTest {
 			.selectors(
 				selectMethod(JUnit5ParameterizedTest.class.getName(), "parameterizedMethod", "java.lang.String,float"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -538,7 +545,7 @@ public class BundleEngineTest {
 		b.selectors(selectMethod(JUnit5ParameterizedTest.class.getName(), "customParameter",
 			"aQute.tester.test.params.CustomParameter"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -555,7 +562,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(containerClass(JUnit4Test.class)))
@@ -571,7 +578,7 @@ public class BundleEngineTest {
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.selectors(selectClass("some.unknown.Clazz"), selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -588,7 +595,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass("some.unknown.Clazz"), selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unresolvedClasses"), finishedSuccessfully()))
@@ -604,7 +611,7 @@ public class BundleEngineTest {
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.selectors(selectMethod("some.unknown.Clazz", "someMethod"), selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -620,7 +627,7 @@ public class BundleEngineTest {
 		engineInFramework()
 			.selectors(selectMethod("some.unknown.Clazz", "someMethod"), selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unresolvedClasses"), finishedSuccessfully()))
@@ -636,7 +643,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectMethod(TestClass.class.getName(), "someMethod"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -652,7 +659,7 @@ public class BundleEngineTest {
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.selectors(selectMethod(TestClass.class.getName(), "someMethod"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -669,7 +676,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -686,7 +693,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(TestClass.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -708,7 +715,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -732,7 +739,7 @@ public class BundleEngineTest {
 		Bundle resolvedTestBundle = startTestBundle(TestClass.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unresolvedClasses")))
@@ -758,7 +765,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass("some.unknown.class"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unresolvedBundles"), finishedSuccessfully()))
@@ -770,7 +777,7 @@ public class BundleEngineTest {
 			.haveExactly(1, event(container("unresolvedClasses"), finishedSuccessfully()));
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unresolvedBundles"), finishedSuccessfully()))
@@ -797,7 +804,7 @@ public class BundleEngineTest {
 		testBundle.start();
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(container("unattachedFragments"), finishedSuccessfully()))
@@ -820,7 +827,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unattachedFragments")))
@@ -845,7 +852,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -876,7 +883,7 @@ public class BundleEngineTest {
 
 		engineInFramework().configurationParameter(CHECK_UNRESOLVED, "false")
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -909,7 +916,7 @@ public class BundleEngineTest {
 		testHost.start();
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(container("unattachedFragments")))
@@ -963,7 +970,7 @@ public class BundleEngineTest {
 		engineInFramework()
 			.selectors(selectBundle(tb1), selectBundle(tb2.getSymbolicName()), selectBundle("test.bundle", "[1,2)"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(containerClass(JUnit4Test.class), finishedSuccessfully()))
@@ -973,7 +980,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectBundle("test.bundle", "[2,3)"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(containerClass(AnotherTestClass.class), finishedSuccessfully()))
@@ -988,7 +995,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectBundle(tb1))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(containerClass(JUnit4Test.class), finishedSuccessfully()))
@@ -998,7 +1005,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectBundle(tb2))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(fragment(tb2), finishedSuccessfully()))
@@ -1021,7 +1028,7 @@ public class BundleEngineTest {
 			.start();
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(containerClass(JUnit4Test.class), finishedSuccessfully()));
@@ -1044,7 +1051,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(JUnit4Test.class.getName()))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(containerClass(JUnit4Test.class), finishedSuccessfully()));
@@ -1065,7 +1072,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(TestClass.class))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(uniqueIdSubstring("host.bundle"), test("thisIsATest"), finishedSuccessfully()))
@@ -1087,7 +1094,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectMethod(JUnit4Test.class, "aTest"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1, event(uniqueIdSubstring("host.bundle"), test("aTest"), finishedSuccessfully()))
@@ -1109,7 +1116,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectMethod(TestClass.class, "thisIsATest"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, event(uniqueIdSubstring("dynamic.bundle"), containerClass(TestClass.class)))
@@ -1122,14 +1129,13 @@ public class BundleEngineTest {
 		Bundle test = startTestBundle(JUnit3And4Test.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
-				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit3And4Test.class),
-					finishedWithFailure(instanceOf(JUnitException.class),
-						message(
-							x -> x.matches("^(?si).*TestCase.*JUnit 4 annotations.*annotations will be ignored.*$")))));
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit3And4Test.class), finishedWithFailure(
+					instanceOf(JUnitException.class),
+					message(x -> x.matches("^(?si).*TestCase.*JUnit 4 annotations.*annotations will be ignored.*$")))));
 
 	}
 
@@ -1139,7 +1145,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectClass(JUnit3And4Test.class))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -1155,7 +1161,7 @@ public class BundleEngineTest {
 
 		engineInFramework().selectors(selectMethod(JUnit3And4Test.class, "testSomething"))
 			.execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(1,
@@ -1173,9 +1179,269 @@ public class BundleEngineTest {
 		Bundle test = startTestBundle(JUnit3And5Test.class, JUnit3AndVenusTest.class);
 
 		engineInFramework().execute()
-			.all()
+			.allEvents()
 			.debug(debugStr)
 			.assertThatEvents()
 			.haveExactly(0, finishedWithFailure(instanceOf(JUnitException.class)));
 	}
+
+	// Tests for https://github.com/bndtools/bnd/issues/3882
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withUnsatisifiedField_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(TestWithField.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(TestWithField.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName() + ".*")))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withClassSelector_withUnsatisifiedField_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(TestWithField.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectClass(TestWithField.class))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(TestWithField.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName() + ".*")))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withMethodSelector_withUnsatisifiedField_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(TestWithField.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectMethod(TestWithField.class, "thisIsATest"))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(TestWithField.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName() + ".*")))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withUnsatisifiedParameter_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName())))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withClassSelector_withUnsatisifiedParameter_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectClass(JUnit5ParameterizedTest.class))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName())))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withMethodSelector_withUnsatisifiedParameter_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectMethod(JUnit5ParameterizedTest.class, "thisIsATest"))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + CustomParameter.class.getName())))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withUnsatisifiedSuperclass_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedSubclassTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedSubclassTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + JUnit5ParameterizedTest.class.getName())))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withClassSelector_withUnsatisifiedSuperclass_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedSubclassTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectClass(JUnit5ParameterizedSubclassTest.class))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedSubclassTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + JUnit5ParameterizedTest.class.getName())))));
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void testClass_withMethodSelector_withUnsatisifiedSuperclass_raisesAnError() {
+		builder.excludeExport("aQute.tester.test.params");
+		startLaunchpad();
+		Bundle test = buildTestBundle(JUnit5ParameterizedSubclassTest.class, TestClass.class)
+			.importPackage("aQute.tester.test.params;resolution:=optional")
+			.start();
+
+		engineInFramework().selectors(selectMethod(JUnit5ParameterizedSubclassTest.class, "thisIsATest"))
+			.execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1,
+				event(uniqueIdSubstring(test.getSymbolicName()), testClass(JUnit5ParameterizedSubclassTest.class),
+					finishedWithFailure(instanceOf(NoClassDefFoundError.class),
+						message(x -> x.matches("^(?si).*" + JUnit5ParameterizedTest.class.getName())))));
+	}
+
+	// Test for https://github.com/bndtools/bnd/issues/3966
+	@Test
+	public void bundle_withSingleTestClass_andItIsNotResolvable_raisesAnError() {
+		Bundle test = testBundler.bundleWithEE()
+			.header("Test-Cases", "some.other.Clazz")
+			.start();
+
+		engineInFramework().execute()
+			.allEvents()
+			.debug(debugStr)
+			.assertThatEvents()
+			.haveExactly(1, event(test("some.other.Clazz"), withParentLastSegment("bundle", test.getSymbolicName()),
+				finishedWithFailure(instanceOf(ClassNotFoundException.class))));
+	}
+
+	public static class SubEngine implements TestEngine {
+
+		static EngineDiscoveryRequest request;
+
+		@Override
+		public String getId() {
+			return "sub-engine";
+		}
+
+		@Override
+		public TestDescriptor discover(EngineDiscoveryRequest discoveryRequest, UniqueId uniqueId) {
+			request = discoveryRequest;
+			return null;
+		}
+
+		@Override
+		public void execute(ExecutionRequest request) {}
+	}
+
+	@Test
+	@Tag(CUSTOM_LAUNCH)
+	public void subRequest_resolvesAndRemovesBundleSelectors() throws Exception {
+		startLaunchpadNoEngines();
+
+		Bundle engineBundle = testBundler.bundleWithEE()
+			.includeResource("META-INF/services/" + TestEngine.class.getName())
+			.literal(SubEngine.class.getName())
+			.addResource(SubEngine.class)
+			.start();
+		Bundle testBundle = startTestBundle(JUnit4Test.class);
+
+		EngineDiscoveryRequest request;
+
+		synchronized (SubEngine.class) {
+			SubEngine.request = null;
+
+			engineInFramework().selectors(selectBundle(testBundle))
+				.execute()
+				.allEvents()
+				.debug(debugStr);
+			request = SubEngine.request;
+		}
+		assertThat(request).isNotNull();
+
+		Class<?> junit4TestInBundle = testBundle.loadClass(JUnit4Test.class.getName());
+
+		List<Class<?>> selectedClasses = new ArrayList<>();
+
+		try (AutoCloseableSoftAssertions softly = new AutoCloseableSoftAssertions()) {
+			softly.assertThat(request.getSelectorsByType(BundleSelector.class))
+				.as("should remove BundleSelectors")
+				.isEmpty();
+			List<DiscoverySelector> selectors = request.getSelectorsByType(DiscoverySelector.class);
+			softly.assertThat(selectors)
+				.as("size")
+				.hasSize(1);
+			List<Class<?>> actualClasses = selectors.stream()
+				.filter(ClassSelector.class::isInstance)
+				.map(ClassSelector.class::cast)
+				.map(ClassSelector::getJavaClass)
+				.collect(Collectors.toList());
+			softly.assertThat(actualClasses)
+				.as("classes")
+				.containsExactlyInAnyOrder(junit4TestInBundle);
+		}
+	}
 }
