@@ -1,6 +1,7 @@
 package org.bndtools.builder;
 
 import java.io.File;
+import java.util.ArrayList;
 import java.util.Collection;
 
 import org.bndtools.build.api.IProjectDecorator.BndProjectInfo;
@@ -10,30 +11,39 @@ import aQute.bnd.osgi.Packages;
 
 public class BndProjectInfoAdapter implements BndProjectInfo {
 
-	private final Project project;
+	final Collection<File>	sourcePath;
+	final Packages			exports;
+	final Packages			imports;
+	final Packages			contained;
 
-	public BndProjectInfoAdapter(Project project) {
-		this.project = project;
+	public BndProjectInfoAdapter(Project project) throws Exception {
+		sourcePath = new ArrayList<>(project.getSourcePath());
+		exports = project.getExports()
+			.dup();
+		imports = project.getImports()
+			.dup();
+		contained = project.getContained()
+			.dup();
 	}
 
 	@Override
 	public Collection<File> getSourcePath() throws Exception {
-		return project.getSourcePath();
+		return sourcePath;
 	}
 
 	@Override
 	public Packages getExports() {
-		return project.getExports();
+		return exports;
 	}
 
 	@Override
 	public Packages getImports() {
-		return project.getImports();
+		return imports;
 	}
 
 	@Override
 	public Packages getContained() {
-		return project.getContained();
+		return contained;
 	}
 
 }
