@@ -7,8 +7,7 @@ import org.eclipse.swt.graphics.Image;
 
 public class PrivatePackageTableLabelProvider extends LabelProvider implements ITableLabelProvider {
 
-	private final Image packageImg = Icons.desc("package")
-		.createImage();
+	private final static Image packageImg = Icons.image("package");
 
 	@Override
 	public Image getColumnImage(Object element, int columnIndex) {
@@ -26,10 +25,4 @@ public class PrivatePackageTableLabelProvider extends LabelProvider implements I
 		}
 		return text;
 	}
-
-	@Override
-	public void dispose() {
-		super.dispose();
-		packageImg.dispose();
-	}
 }
