@@ -1,5 +1,8 @@
 package org.bndtools.core.resolve;
 
+import static java.util.Objects.requireNonNull;
+
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
@@ -8,13 +11,15 @@ import org.osgi.resource.Resource;
 import org.osgi.resource.Wire;
 import org.osgi.service.resolver.ResolutionException;
 
+import biz.aQute.resolve.ResolverLogger;
 import biz.aQute.resolve.RunResolution;
 
 public class ResolutionResult {
 
-	private final Outcome		outcome;
-	private final IStatus		status;
-	private final RunResolution	resolution;
+	private final Outcome			outcome;
+	private final IStatus			status;
+	private final RunResolution		resolution;
+	private final ResolverLogger	logger;
 
 	public enum Outcome {
 		Resolved,
@@ -23,10 +28,11 @@ public class ResolutionResult {
 		Cancelled
 	}
 
-	public ResolutionResult(Outcome outcome, RunResolution resolution, IStatus status) {
-		this.outcome = outcome;
+	public ResolutionResult(Outcome outcome, RunResolution resolution, IStatus status, ResolverLogger logger) {
+		this.outcome = requireNonNull(outcome);
 		this.resolution = resolution;
-		this.status = status;
+		this.status = requireNonNull(status);
+		this.logger = requireNonNull(logger);
 	}
 
 	public Outcome getOutcome() {
@@ -34,16 +40,23 @@ public class ResolutionResult {
 	}
 
 	public Map<Resource, List<Wire>> getResourceWirings() {
-		return resolution.required;
+		if (resolution != null) {
+			return resolution.required;
+		}
+		return Collections.emptyMap();
 	}
 
 	public Map<Resource, List<Wire>> getOptionalResources() {
-		return resolution.optional;
+		if (resolution != null) {
+			return resolution.optional;
+		}
+		return Collections.emptyMap();
 	}
 
 	public ResolutionException getResolutionException() {
-		if (resolution.exception instanceof ResolutionException)
+		if ((resolution != null) && (resolution.exception instanceof ResolutionException)) {
 			return (ResolutionException) resolution.exception;
+		}
 		return null;
 	}
 
@@ -52,11 +65,17 @@ public class ResolutionResult {
 	}
 
 	public String getLog() {
-		return resolution.log;
+		if (resolution != null) {
+			return resolution.log;
+		}
+		return "";
 	}
 
 	public RunResolution getResolution() {
 		return resolution;
 	}
 
+	public ResolverLogger getLogger() {
+		return logger;
+	}
 }
