@@ -1,5 +1,3 @@
-package aQute.bnd.maven.plugin;
-
 /*
  * Copyright (c) Paremus and others (2015, 2016). All Rights Reserved.
  *
@@ -16,11 +14,18 @@ package aQute.bnd.maven.plugin;
  * limitations under the License.
  */
 
+package aQute.bnd.maven.plugin;
+
+import static aQute.bnd.maven.lib.executions.PluginExecutions.isPackagingGoal;
+
 import java.io.File;
+import java.io.IOException;
 import java.io.OutputStream;
 import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
@@ -30,10 +35,12 @@ import java.util.Properties;
 import java.util.Set;
 import java.util.jar.Manifest;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
 
 import org.apache.maven.artifact.Artifact;
+import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
 import org.apache.maven.model.Developer;
 import org.apache.maven.model.License;
 import org.apache.maven.model.Plugin;
@@ -46,6 +53,7 @@ import org.apache.maven.plugin.MojoFailureException;
 import org.apache.maven.plugins.annotations.Component;
 import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.project.MavenProject;
+import org.apache.maven.project.MavenProjectHelper;
 import org.apache.maven.settings.Settings;
 import org.apache.maven.shared.mapping.MappingUtils;
 import org.codehaus.plexus.util.StringUtils;
@@ -55,7 +63,9 @@ import org.slf4j.LoggerFactory;
 import org.sonatype.plexus.build.incremental.BuildContext;
 
 import aQute.bnd.build.Project;
+import aQute.bnd.exceptions.Exceptions;
 import aQute.bnd.header.OSGiHeader;
+import aQute.bnd.maven.PomPropertiesResource;
 import aQute.bnd.maven.lib.configuration.BeanProperties;
 import aQute.bnd.osgi.Builder;
 import aQute.bnd.osgi.Constants;
@@ -71,34 +81,40 @@ import aQute.lib.utf8properties.UTF8Properties;
 import aQute.service.reporter.Report.Location;
 
 public abstract class AbstractBndMavenPlugin extends AbstractMojo {
-	private static final Logger						logger					= LoggerFactory
-		.getLogger(AbstractBndMavenPlugin.class);
-	static final String						MANIFEST_LAST_MODIFIED	= "aQute.bnd.maven.plugin.BndMavenPlugin.manifestLastModified";
-	static final String						MARKED_FILES			= "aQute.bnd.maven.plugin.BndMavenPlugin.markedFiles";
-	static final String						PACKAGING_JAR			= "jar";
-	static final String						PACKAGING_WAR			= "war";
-	static final String						TSTAMP					= "${tstamp}";
+	protected final Logger	logger					= LoggerFactory.getLogger(getClass());
+	static final String		MANIFEST_LAST_MODIFIED	= "aQute.bnd.maven.plugin.BndMavenPlugin.manifestLastModified";
+	static final String		MARKED_FILES			= "aQute.bnd.maven.plugin.BndMavenPlugin.markedFiles";
+	static final String		PACKAGING_JAR			= "jar";
+	static final String		PACKAGING_WAR			= "war";
+	static final String		TSTAMP					= "${tstamp}";
+	static final String		SNAPSHOT				= "SNAPSHOT";
 
 	@Parameter(defaultValue = "${project.build.directory}", readonly = true)
-	File									targetDir;
+	File					targetDir;
 
 	@Parameter(defaultValue = "true")
-	boolean									includeClassesDir;
+	boolean					includeClassesDir;
 
 	@Parameter(defaultValue = "${project.build.directory}/${project.build.finalName}")
-	File									warOutputDir;
+	File					warOutputDir;
 
 	@Parameter(defaultValue = "${project}", required = true, readonly = true)
-	MavenProject							project;
+	MavenProject			project;
 
 	@Parameter(defaultValue = "${settings}", readonly = true)
-	Settings								settings;
+	Settings				settings;
 
 	@Parameter(defaultValue = "${mojoExecution}", readonly = true)
-	MojoExecution							mojoExecution;
+	MojoExecution			mojoExecution;
 
 	@Parameter(property = "bnd.packagingTypes", defaultValue = PACKAGING_JAR + "," + PACKAGING_WAR)
-	List<String>							packagingTypes;
+	List<String>			packagingTypes;
+
+	@Parameter(property = "bnd.skipIfEmpty", defaultValue = "false")
+	boolean					skipIfEmpty;
+
+	@Parameter(defaultValue = "${project.build.outputTimestamp}")
+	private String			outputTimestamp;
 
 	/**
 	 * File path to a bnd file containing bnd instructions for this project.
@@ -111,7 +127,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	@Parameter(defaultValue = Project.BNDFILE)
 	// This is not used and is for doc only; see loadProjectProperties
 	@SuppressWarnings("unused")
-	String									bndfile;
+	String					bndfile;
 
 	/**
 	 * Bnd instructions for this project specified directly in the pom file.
@@ -125,12 +141,15 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 	@Parameter
 	// This is not used and is for doc only; see loadProjectProperties
 	@SuppressWarnings("unused")
-	String									bnd;
+	String					bnd;
 
 	@Component
-	BuildContext							buildContext;
+	BuildContext			buildContext;
 
-	File									propertiesFile;
+	@Component
+	MavenProjectHelper		projectHelper;
+
+	File					propertiesFile;
 
 	public abstract File getSourceDir();
 
@@ -144,17 +163,33 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 
 	public abstract boolean isSkip();
 
+	public Optional<String> getClassifier() {
+		return Optional.empty();
+	}
+
+	public Optional<String> getType() {
+		return Optional.empty();
+	}
+
 	@Override
 	public void execute() throws MojoExecutionException, MojoFailureException {
-		if (isSkip()) {
-			logger.debug("skip project as configured");
-			return;
-		}
-
 		// Exit without generating anything if this is neither a jar or war
 		// project. Probably it's just a parent project.
 		if (!packagingTypes.contains(project.getPackaging())) {
-			logger.info("skip project with packaging=" + project.getPackaging());
+			logger.debug("skip project with packaging=" + project.getPackaging());
+			return;
+		}
+
+		if (isSkip()) {
+			logger.info("skip project as configured");
+			return;
+		}
+
+		File classesDir = getClassesDir();
+
+		if (skipIfEmpty && includeClassesDir && isEmpty(classesDir)) {
+			logger.info(
+				"skip project because includeClassesDir=true, compiler output directory is empty and skipIfEmpty=true");
 			return;
 		}
 
@@ -162,7 +197,11 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		beanProperties.put("project", project);
 		beanProperties.put("settings", settings);
 		Properties mavenProperties = new Properties(beanProperties);
-		mavenProperties.putAll(project.getProperties());
+		Properties projectProperties = project.getProperties();
+		for (Enumeration<?> propertyNames = projectProperties.propertyNames(); propertyNames.hasMoreElements();) {
+			Object key = propertyNames.nextElement();
+			mavenProperties.put(key, projectProperties.get(key));
+		}
 
 		try (Builder builder = new Builder(new Processor(mavenProperties, false))) {
 			builder.setTrace(logger.isDebugEnabled());
@@ -189,9 +228,11 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			if (getClassesDir().isDirectory()) {
 				builder.addClasspath(getClassesDir());
 
-				Jar classesDirJar = new Jar(project.getName(), getClassesDir());
-				if (!includeClassesDir) {
-					classesDirJar.removePrefix(""); // clear the jar
+				Jar classesDirJar;
+				if (includeClassesDir) {
+					classesDirJar = new Jar(project.getName(), getClassesDir());
+				} else {
+					classesDirJar = new Jar(project.getName()); // empty jar
 				}
 				classesDirJar.setManifest(new Manifest());
 				builder.setJar(classesDirJar);
@@ -220,6 +261,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 			Set<Artifact> artifacts = project.getArtifacts();
 			List<Object> buildpath = new ArrayList<>(artifacts.size());
 			List<String> wablibs = new ArrayList<>(artifacts.size());
+			final ScopeArtifactFilter scopeFilter = new ScopeArtifactFilter(Artifact.SCOPE_RUNTIME);
 			for (Artifact artifact : artifacts) {
 				File cpe = artifact.getFile()
 					.getCanonicalFile();
@@ -229,6 +271,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 				}
 				if (cpe.isDirectory()) {
 					Jar cpeJar = new Jar(cpe);
+					cpeJar.setSource(new File(cpe.getParentFile(), createArtifactName(artifact)));
 					builder.addClose(cpeJar);
 					builder.updateModified(cpeJar.lastModified(), cpe.getPath());
 					buildpath.add(cpeJar);
@@ -250,9 +293,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 					builder.updateModified(cpe.lastModified(), cpe.getPath());
 					buildpath.add(cpe);
 
-					if (isWab && !hasWablibs && !Artifact.SCOPE_PROVIDED.equals(artifact.getScope())
-						&& !Artifact.SCOPE_TEST.equals(artifact.getScope()) && !artifact.isOptional()) {
-
+					if (isWab && !hasWablibs && !artifact.isOptional() && scopeFilter.include(artifact)) {
 						String fileNameMapping = MappingUtils
 							.evaluateFileNameMapping(MappingUtils.DEFAULT_FILE_NAME_MAPPING, artifact);
 						wablibs.add("WEB-INF/lib/" + fileNameMapping + "=" + cpe.getName() + ";lib:=true");
@@ -290,6 +331,18 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 
 			processBuilder(builder);
 
+			// https://maven.apache.org/guides/mini/guide-reproducible-builds.html
+			boolean isReproducible = Strings.nonNullOrEmpty(outputTimestamp)
+				// no timestamp configured (1 character configuration is useful
+				// to override a full value during pom inheritance)
+				&& ((outputTimestamp.length() > 1) || Character.isDigit(outputTimestamp.charAt(0)));
+			if (isReproducible) {
+				builder.setProperty(Constants.REPRODUCIBLE, outputTimestamp);
+				if (builder.getProperty(Constants.NOEXTRAHEADERS) == null) {
+					builder.setProperty(Constants.NOEXTRAHEADERS, Boolean.TRUE.toString());
+				}
+			}
+
 			// Set Bundle-SymbolicName
 			if (builder.getProperty(Constants.BUNDLE_SYMBOLICNAME) == null) {
 				builder.setProperty(Constants.BUNDLE_SYMBOLICNAME, project.getArtifactId());
@@ -299,11 +352,17 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 				builder.setProperty(Constants.BUNDLE_NAME, project.getName());
 			}
 			// Set Bundle-Version
+			String snapshot = isReproducible ? SNAPSHOT : null;
 			if (builder.getProperty(Constants.BUNDLE_VERSION) == null) {
 				Version version = new MavenVersion(project.getVersion()).getOSGiVersion();
 				builder.setProperty(Constants.BUNDLE_VERSION, version.toString());
+				if (snapshot == null) {
+					snapshot = TSTAMP;
+				}
+			}
+			if (snapshot != null) {
 				if (builder.getProperty(Constants.SNAPSHOT) == null) {
-					builder.setProperty(Constants.SNAPSHOT, TSTAMP);
+					builder.setProperty(Constants.SNAPSHOT, snapshot);
 				}
 			}
 
@@ -437,8 +496,31 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 				// Build bnd Jar (in memory)
 				Jar bndJar = builder.build();
 
-				// Expand Jar into target/classes
-				expandJar(bndJar, outputDir);
+				String goal = mojoExecution.getMojoDescriptor()
+					.getGoal();
+
+				// If a bnd-maven-plugin packaging goal is used it means
+				// this execution is responsible for creating and attaching the
+				// target artifact to the project
+				if (isPackagingGoal(goal)) {
+					// However, if extensions for this plugin are not enabled
+					// the maven-(j|w)wa-plugin will also execute, resulting in
+					// conflicting results
+					if (mojoExecution.getPlugin()
+						.isExtensions()) {
+						// Add META-INF/maven metadata to jar
+						addMavenMetadataToJar(bndJar);
+						// Write the jar directly and attach it to the project
+						attachArtifactToProject(bndJar);
+					} else {
+						throw new MojoExecutionException(String.format(
+							"In order to use the bnd-maven-plugin packaging goal %s, <extensions>true</extensions> must be set on the plugin",
+							goal));
+					}
+				} else {
+					// Expand Jar into target/classes
+					expandJar(bndJar, outputDir);
+				}
 			} else {
 				logger.debug("No build");
 			}
@@ -490,6 +572,71 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		return builder;
 	}
 
+	private void attachArtifactToProject(Jar bndJar) throws Exception {
+		File artifactFile = createArtifactFile();
+		File outputDir = artifactFile.getParentFile();
+
+		if (!outputDir.exists()) {
+			IO.mkdirs(outputDir);
+		}
+
+		try (OutputStream os = buildContext.newFileOutputStream(artifactFile)) {
+			bndJar.write(os);
+		}
+
+		// If there is a classifier artifact must be attached to the
+		// project using that classifier
+		Optional<String> classifier = getClassifier();
+		if (classifier.isPresent()) {
+			projectHelper.attachArtifact(project, getType().orElse(""), classifier.get(), artifactFile);
+		} else {
+			// If there is no classifier, then this artifact is the main
+			// artifact
+
+			Artifact artifact = project.getArtifact();
+			if (Optional.ofNullable(artifact.getFile())
+				.map(File::isFile)
+				.orElse(Boolean.FALSE)) {
+
+				logger.warn("The main artifact on {} was already set. It will be replaced by {}", project,
+					mojoExecution);
+			}
+			artifact.setFile(artifactFile);
+		}
+	}
+
+	private void addMavenMetadataToJar(Jar bndJar) throws IOException {
+		String groupId = project.getGroupId();
+		String artifactId = project.getArtifactId();
+		String version = project.getArtifact()
+			.isSnapshot()
+				? project.getArtifact()
+					.getVersion()
+				: project.getVersion();
+
+		bndJar.putResource(String.format("META-INF/maven/%s/%s/pom.xml", groupId, artifactId),
+			new FileResource(project.getFile()));
+		PomPropertiesResource pomProperties = new PomPropertiesResource(groupId, artifactId, version);
+		bndJar.putResource(pomProperties.getWhere(), pomProperties);
+	}
+
+	private File createArtifactFile() {
+		return new File(targetDir, project.getBuild()
+			.getFinalName()
+			+ getClassifier().map("-"::concat)
+				.orElse("")
+			+ "." + project.getPackaging());
+	}
+
+	private String createArtifactName(Artifact artifact) {
+		String classifier = artifact.getClassifier();
+		if ((classifier == null) || classifier.isEmpty()) {
+			return String.format("%s-%s.%s", artifact.getArtifactId(), artifact.getVersion(), artifact.getType());
+		}
+		return String.format("%s-%s-%s.%s", artifact.getArtifactId(), artifact.getVersion(), classifier,
+			artifact.getType());
+	}
+
 	private File loadProperties(Builder builder) throws Exception {
 		// Load parent project properties first
 		loadParentProjectProperties(builder, project);
@@ -581,7 +728,7 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		return new Xpp3Dom("configuration");
 	}
 
-	private void reportErrorsAndWarnings(Builder builder) throws MojoFailureException {
+	protected void reportErrorsAndWarnings(Builder builder) throws MojoFailureException {
 		@SuppressWarnings("unchecked")
 		Collection<File> markedFiles = (Collection<File>) buildContext.getValue(MARKED_FILES);
 		if (markedFiles == null) {
@@ -628,6 +775,23 @@ public abstract class AbstractBndMavenPlugin extends AbstractMojo {
 		}
 	}
 
+	protected boolean isEmpty(File directory) {
+		if (directory == null || !directory.isDirectory()) {
+			return true;
+		}
+
+		Path path = directory.toPath();
+		try (Stream<Path> entries = Files.walk(path)) {
+			return !entries.filter(p -> !Files.isDirectory(p))
+				.filter(p -> !path.relativize(p)
+					.startsWith("META-INF/"))
+				.findFirst()
+				.isPresent();
+		} catch (IOException ioe) {
+			throw Exceptions.duck(ioe);
+		}
+	}
+
 	private void expandJar(Jar jar, File dir) throws Exception {
 		final long lastModified = jar.lastModified();
 		if (logger.isDebugEnabled()) {
